@@ -291,7 +291,7 @@ class Router(object):
 
 
 class Request(webob.Request):
-    """Add some Openstack API-specific logic to the base webob.Request."""
+    """Add some OpenStack API-specific logic to the base webob.Request."""
 
     default_request_content_types = ('application/json', 'application/xml')
     default_accept_types = ('application/json', 'application/xml')
@@ -471,7 +471,7 @@ class XMLDictSerializer(DictSerializer):
 
     def default(self, data):
         # We expect data to contain a single key which is the XML root.
-        root_key = data.keys()[0]
+        root_key = list(data.keys())[0]
         doc = minidom.Document()
         node = self._to_xml_node(doc, self.metadata, root_key, data[root_key])
 
@@ -659,23 +659,23 @@ class RequestDeserializer(object):
 
     def deserialize_body(self, request, action):
         if not len(request.body) > 0:
-            LOG.debug(_("Empty body provided in request"))
+            LOG.debug("Empty body provided in request")
             return {}
 
         try:
             content_type = request.get_content_type()
         except base_exception.InvalidContentType:
-            LOG.debug(_("Unrecognized Content-Type provided in request"))
+            LOG.debug("Unrecognized Content-Type provided in request")
             raise
 
         if content_type is None:
-            LOG.debug(_("No Content-Type provided in request"))
+            LOG.debug("No Content-Type provided in request")
             return {}
 
         try:
             deserializer = self.get_body_deserializer(content_type)
         except base_exception.InvalidContentType:
-            LOG.debug(_("Unable to deserialize body as provided Content-Type"))
+            LOG.debug("Unable to deserialize body as provided Content-Type")
             raise
 
         return deserializer.deserialize(request.body, action)
