@@ -22,6 +22,8 @@ from trove.cluster.views import ClusterView
 from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
+from trove.common.notification import DBaaSClusterGrow
+from trove.common.notification import StartNotification
 from trove.common import remote
 from trove.common.strategies.cluster import base
 from trove.common import utils
@@ -42,74 +44,6 @@ class MongoDbAPIStrategy(base.BaseAPIStrategy):
     def cluster_class(self):
         return MongoDbCluster
 
-    @property
-    def cluster_controller_actions(self):
-        return {'add_shard': self._action_add_shard,
-                'grow': self._action_grow,
-                'shrink': self._action_shrink}
-
-    def _action_add_shard(self, cluster, body):
-        cluster.add_shard()
-
-    def _action_grow(self, cluster, body):
-        cluster.grow(self._parse_grow_schema(body))
-
-    def _action_shrink(self, cluster, body):
-        instance_ids = [instance['id'] for instance in body['shrink']]
-        cluster.shrink(instance_ids)
-
-    def _parse_grow_schema(self, body):
-        items = body['grow']
-        instances = []
-        for item in items:
-            instances.append(self._parse_grow_item(item))
-        return instances
-
-    def _parse_grow_item(self, item):
-        used_keys = []
-
-        def _check_option(key, required=False, valid_values=None):
-            if required and key not in item:
-                raise exception.TroveError(
-                    _('An instance with the options %(given)s is missing '
-                      'the MongoDB required option %(expected)s.')
-                    % {'given': item.keys(), 'expected': key}
-                )
-            value = item.get(key, None)
-            if valid_values and value not in valid_values:
-                raise exception.TroveError(
-                    _('The value %(value)s for key %(key)s is invalid. '
-                      'Allowed values are %(valid)s.')
-                    % {'value': value, 'key': key, 'valid': valid_values}
-                )
-            used_keys.append(key)
-            return value
-
-        flavor_id = utils.get_id_from_href(_check_option('flavorRef',
-                                                         required=True))
-        volume_size = int(_check_option('volume', required=True)['size'])
-        instance_type = _check_option('type', required=True,
-                                      valid_values=['replica',
-                                                    'query_router'])
-        name = _check_option('name')
-        related_to = _check_option('related_to')
-
-        unused_keys = list(set(item.keys()).difference(set(used_keys)))
-        if unused_keys:
-            raise exception.TroveError(
-                _('The arguments %s are not supported by MongoDB.')
-                % unused_keys
-            )
-
-        instance = {'flavor_id': flavor_id,
-                    'volume_size': volume_size,
-                    'instance_type': instance_type}
-        if name:
-            instance['name'] = name
-        if related_to:
-            instance['related_to'] = related_to
-        return instance
-
     @property
     def cluster_view_class(self):
         return MongoDbClusterView
@@ -181,13 +115,24 @@ class MongoDbCluster(models.Cluster):
             task_status=ClusterTasks.BUILDING_INITIAL)
 
         replica_set_name = "rs1"
-        key = utils.generate_random_password()
 
         member_config = {"id": db_info.id,
                          "shard_id": utils.generate_uuid(),
                          "instance_type": "member",
-                         "replica_set_name": replica_set_name,
-                         "key": key}
+                         "replica_set_name": replica_set_name}
+
+        configsvr_config = {"id": db_info.id,
+                            "instance_type": "config_server"}
+
+        mongos_config = {"id": db_info.id,
+                         "instance_type": "query_router"}
+
+        if mongo_conf.cluster_secure:
+            cluster_key = utils.generate_random_password()
+            member_config['key'] = cluster_key
+            configsvr_config['key'] = cluster_key
+            mongos_config['key'] = cluster_key
+
         for i in range(0, num_instances):
             instance_name = "%s-%s-%s" % (name, replica_set_name, str(i + 1))
             inst_models.Instance.create(context, instance_name,
@@ -201,9 +146,6 @@ class MongoDbCluster(models.Cluster):
                                         configuration_id=None,
                                         cluster_config=member_config)
 
-        configsvr_config = {"id": db_info.id,
-                            "instance_type": "config_server",
-                            "key": key}
         for i in range(1, num_configsvr + 1):
             instance_name = "%s-%s-%s" % (name, "configsvr", str(i))
             inst_models.Instance.create(context, instance_name,
@@ -217,9 +159,6 @@ class MongoDbCluster(models.Cluster):
                                         configuration_id=None,
                                         cluster_config=configsvr_config)
 
-        mongos_config = {"id": db_info.id,
-                         "instance_type": "query_router",
-                         "key": key}
         for i in range(1, num_mongos + 1):
             instance_name = "%s-%s-%s" % (name, "mongos", str(i))
             inst_models.Instance.create(context, instance_name,
@@ -238,6 +177,64 @@ class MongoDbCluster(models.Cluster):
 
         return MongoDbCluster(context, db_info, datastore, datastore_version)
 
+    def _parse_grow_item(self, item):
+        used_keys = []
+
+        def _check_option(key, required=False, valid_values=None):
+            if required and key not in item:
+                raise exception.TroveError(
+                    _('An instance with the options %(given)s is missing '
+                      'the MongoDB required option %(expected)s.')
+                    % {'given': item.keys(), 'expected': key}
+                )
+            value = item.get(key, None)
+            if valid_values and value not in valid_values:
+                raise exception.TroveError(
+                    _('The value %(value)s for key %(key)s is invalid. '
+                      'Allowed values are %(valid)s.')
+                    % {'value': value, 'key': key, 'valid': valid_values}
+                )
+            used_keys.append(key)
+            return value
+
+        flavor_id = utils.get_id_from_href(_check_option('flavorRef',
+                                                         required=True))
+        volume_size = int(_check_option('volume', required=True)['size'])
+        instance_type = _check_option('type', required=True,
+                                      valid_values=['replica',
+                                                    'query_router'])
+        name = _check_option('name')
+        related_to = _check_option('related_to')
+
+        unused_keys = list(set(item.keys()).difference(set(used_keys)))
+        if unused_keys:
+            raise exception.TroveError(
+                _('The arguments %s are not supported by MongoDB.')
+                % unused_keys
+            )
+
+        instance = {'flavor_id': flavor_id,
+                    'volume_size': volume_size,
+                    'instance_type': instance_type}
+        if name:
+            instance['name'] = name
+        if related_to:
+            instance['related_to'] = related_to
+        return instance
+
+    def action(self, context, req, action, param):
+        if action == 'grow':
+            context.notification = DBaaSClusterGrow(context, request=req)
+            with StartNotification(context, cluster_id=self.id):
+                return self.grow([self._parse_grow_item(item)
+                                  for item in param])
+        elif action == 'add_shard':
+            context.notification = DBaaSClusterGrow(context, request=req)
+            with StartNotification(context, cluster_id=self.id):
+                return self.add_shard()
+        else:
+            super(MongoDbCluster, self).action(context, req, action, param)
+
     def add_shard(self):
 
         if self.db_info.task_status != ClusterTasks.NONE:
@@ -347,23 +344,23 @@ class MongoDbCluster(models.Cluster):
         target_ids = set(instance_ids)
         target_member_ids = target_ids.intersection(all_member_ids)
         target_query_router_ids = target_ids.intersection(all_query_router_ids)
-        other_ids = target_ids.difference(
+        target_configsvr_ids = target_ids.difference(
             target_member_ids.union(target_query_router_ids)
         )
-        if other_ids:
-            raise exception.TroveError(
-                _('Instances %s cannot be deleted. MongoDB cluster shink only '
-                  'supports removing replicas and query routers.')
-                % list(other_ids)
+        if target_configsvr_ids:
+            raise exception.ClusterShrinkInstanceInUse(
+                id=list(target_configsvr_ids),
+                reason="Cannot remove config servers."
             )
 
         remaining_query_router_ids = all_query_router_ids.difference(
             target_query_router_ids
         )
         if len(remaining_query_router_ids) < 1:
-            raise exception.TroveError(
-                _('Cannot delete all remaining query routers. At least one '
-                  'query router must be available in the cluster.')
+            raise exception.ClusterShrinkInstanceInUse(
+                id=list(target_query_router_ids),
+                reason="Cannot remove all remaining query routers. At least "
+                       "one query router must be available in the cluster."
             )
 
         if target_member_ids:
@@ -406,9 +403,10 @@ class MongoDbCluster(models.Cluster):
                           default_name_tag, key=None):
         """Loop through the instances and create them in this cluster."""
         cluster_config['id'] = self.id
-        if not key:
-            key = self.get_guest(self.arbitrary_query_router).get_key()
-        cluster_config['key'] = key
+        if CONF.get(self.datastore_version.manager).cluster_secure:
+            if not key:
+                key = self.get_guest(self.arbitrary_query_router).get_key()
+            cluster_config['key'] = key
         instance_ids = []
         for i, instance in enumerate(instances):
             name = instance.get('name', '%s-%s-%s' % (
@@ -418,6 +416,8 @@ class MongoDbCluster(models.Cluster):
                 self.datastore_version.image_id, [], [],
                 self.datastore, self.datastore_version,
                 instance['volume_size'], None,
+                availability_zone=instance.get('availability_zone', None),
+                nics=instance.get('nics', None),
                 cluster_config=cluster_config
             )
             instance_ids.append(new_instance.id)
