@@ -15,10 +15,11 @@
 
 import abc
 import ast
+import base64
 import csv
 import json
+import re
 import six
-import StringIO
 import yaml
 
 from ConfigParser import SafeConfigParser
@@ -65,8 +66,12 @@ class StringConverter(object):
         return str(value)
 
     def _to_object(self, value):
+        # Return known mappings and quoted strings right away.
         if value in self._object_mappings:
             return self._object_mappings[value]
+        elif (isinstance(value, basestring) and
+              re.match("^'(.*)'|\"(.*)\"$", value)):
+            return value
 
         try:
             return ast.literal_eval(value)
@@ -193,7 +198,7 @@ class IniCodec(StreamCodec):
 
     def serialize(self, dict_data):
         parser = self._init_config_parser(dict_data)
-        output = StringIO.StringIO()
+        output = six.StringIO()
         parser.write(output)
 
         return output.getvalue()
@@ -207,8 +212,8 @@ class IniCodec(StreamCodec):
                 for s in parser.sections()}
 
     def _pre_parse(self, stream):
-        buf = StringIO.StringIO()
-        for line in StringIO.StringIO(stream):
+        buf = six.StringIO()
+        for line in six.StringIO(stream):
             # Ignore commented lines.
             if not line.startswith(self._comment_markers):
                 # Strip leading and trailing whitespaces from each line.
@@ -252,9 +257,10 @@ class PropertiesCodec(StreamCodec):
 
     QUOTING_MODE = csv.QUOTE_MINIMAL
     STRICT_MODE = False
+    SKIP_INIT_SPACE = True
 
     def __init__(self, delimiter=' ', comment_markers=('#'),
-                 unpack_singletons=True, string_mappings={}):
+                 unpack_singletons=True, string_mappings=None):
         """
         :param delimiter:         A one-character used to separate fields.
         :type delimiter:          string
@@ -275,41 +281,46 @@ class PropertiesCodec(StreamCodec):
         """
         self._delimiter = delimiter
         self._comment_markers = comment_markers
-        self._string_converter = StringConverter(string_mappings)
+        self._string_converter = StringConverter(string_mappings or {})
         self._unpack_singletons = unpack_singletons
 
     def serialize(self, dict_data):
-        output = StringIO.StringIO()
+        output = six.StringIO()
         writer = csv.writer(output, delimiter=self._delimiter,
                             quoting=self.QUOTING_MODE,
-                            strict=self.STRICT_MODE)
+                            strict=self.STRICT_MODE,
+                            skipinitialspace=self.SKIP_INIT_SPACE)
 
-        for key, value in sorted(dict_data.items()):
+        for key, value in dict_data.items():
             writer.writerows(self._to_rows(key, value))
 
         return output.getvalue()
 
     def deserialize(self, stream):
-        reader = csv.reader(StringIO.StringIO(stream),
+        reader = csv.reader(six.StringIO(stream),
                             delimiter=self._delimiter,
                             quoting=self.QUOTING_MODE,
-                            strict=self.STRICT_MODE)
+                            strict=self.STRICT_MODE,
+                            skipinitialspace=self.SKIP_INIT_SPACE)
 
         return self._to_dict(reader)
 
     def _to_dict(self, reader):
         data_dict = {}
         for row in reader:
-            # Ignore comment lines.
-            if row and not row[0].startswith(self._comment_markers):
-                items = self._string_converter.to_objects(
-                    [v if v else None for v in row[1:]])
-                current = data_dict.get(row[0])
-                if current is not None:
-                    current.append(trove_utils.unpack_singleton(items)
-                                   if self._unpack_singletons else items)
-                else:
-                    data_dict.update({row[0]: [items]})
+            if row:
+                key = row[0].strip()
+                # Ignore comment lines.
+                if not key.strip().startswith(self._comment_markers):
+                    items = self._string_converter.to_objects(
+                        [v if v else None for v in
+                         map(self._strip_comments, row[1:])])
+                    current = data_dict.get(key)
+                    if current is not None:
+                        current.append(trove_utils.unpack_singleton(items)
+                                       if self._unpack_singletons else items)
+                    else:
+                        data_dict.update({key: [items]})
 
         if self._unpack_singletons:
             # Unpack singleton values.
@@ -318,6 +329,12 @@ class PropertiesCodec(StreamCodec):
 
         return data_dict
 
+    def _strip_comments(self, value):
+        # Strip in-line comments.
+        for marker in self._comment_markers:
+            value = value.split(marker)[0]
+        return value.strip()
+
     def _to_rows(self, header, items):
         rows = []
         if trove_utils.is_collection(items):
@@ -331,7 +348,9 @@ class PropertiesCodec(StreamCodec):
                     header, self._string_converter.to_strings(items)))
         else:
             # This is a single-row property with only one argument.
-            rows.append(self._to_list(header, items))
+            rows.append(
+                self._string_converter.to_strings(
+                    self._to_list(header, items)))
 
         return rows
 
@@ -348,10 +367,59 @@ class PropertiesCodec(StreamCodec):
         return container
 
 
+class KeyValueCodec(PropertiesCodec):
+    """
+    Read/write data from/into a simple key=value file.
+
+    key1=value1
+    key2=value2
+    key3=value3
+    ...
+
+    The above file content would be represented as:
+    {'key1': 'value1',
+     'key2': 'value2',
+     'key3': 'value3',
+     ...
+    }
+    """
+    def __init__(self, delimiter='=', comment_markers=('#'),
+                 unpack_singletons=True, string_mappings=None):
+        super(KeyValueCodec, self).__init__(
+            delimiter=delimiter, comment_markers=comment_markers,
+            unpack_singletons=unpack_singletons,
+            string_mappings=string_mappings)
+
+
 class JsonCodec(StreamCodec):
 
     def serialize(self, dict_data):
         return json.dumps(dict_data)
 
     def deserialize(self, stream):
-        return json.load(StringIO.StringIO(stream))
+        return json.load(six.StringIO(stream))
+
+
+class Base64Codec(StreamCodec):
+    """Serialize (encode) and deserialize (decode) using the base64 codec.
+    To read binary data from a file and b64encode it, used the decode=False
+    flag on operating_system's read calls.  Use encode=False to decode
+    binary data before writing to a file as well.
+    """
+
+    def serialize(self, data):
+
+        try:
+            # py27str - if we've got text data, this should encode it
+            # py27aa/py34aa - if we've got a bytearray, this should work too
+            encoded = str(base64.b64encode(data).decode('utf-8'))
+        except TypeError:
+            # py34str - convert to bytes first, then we can encode
+            data_bytes = bytes([ord(item) for item in data])
+            encoded = base64.b64encode(data_bytes).decode('utf-8')
+        return encoded
+
+    def deserialize(self, stream):
+
+        # py27 & py34 seem to understand bytearray the same
+        return bytearray([item for item in base64.b64decode(stream)])
