@@ -48,10 +48,23 @@ bool_from_string = strutils.bool_from_string
 execute = processutils.execute
 isotime = timeutils.isotime
 
-ENV = jinja2.Environment(loader=jinja2.ChoiceLoader([
-                         jinja2.FileSystemLoader(CONF.template_path),
-                         jinja2.PackageLoader("trove", "templates")
-                         ]))
+
+def build_jinja_environment():
+    env = jinja2.Environment(loader=jinja2.ChoiceLoader([
+        jinja2.FileSystemLoader(CONF.template_path),
+        jinja2.PackageLoader("trove", "templates")
+    ]))
+    # Add some basic operation not built-in.
+    env.globals['max'] = max
+    env.globals['min'] = min
+    return env
+
+ENV = build_jinja_environment()
+
+
+def pagination_limit(limit, default_limit):
+    limit = int(limit or default_limit)
+    return min(limit, default_limit)
 
 
 def create_method_args_string(*args, **kwargs):
@@ -127,7 +140,7 @@ class cached_property(object):
         self.__name__ = name or func.__name__
         self.__doc__ = doc or func.__doc__
 
-    def __get__(self, obj, type=None):
+    def __get__(self, obj, owner):
         if obj is None:
             return self
         value = self.func(obj)
@@ -187,7 +200,7 @@ def build_polling_task(retriever, condition=lambda value: value,
             raise exception.PollTimeOut
 
     return loopingcall.FixedIntervalLoopingCall(
-        f=poll_and_check).start(sleep_time, True)
+        f=poll_and_check).start(sleep_time, initial_delay=False)
 
 
 def poll_until(retriever, condition=lambda value: value,
