@@ -17,6 +17,7 @@ from oslo_log import log as logging
 
 from trove.common import cfg
 from trove.common import exception
+from trove.common.i18n import _
 from trove.common import remote
 from trove.common import utils
 from trove.extensions.mysql import models as mysql_models
@@ -37,7 +38,7 @@ def load_mgmt_instances(context, deleted=None, client=None,
         mgmt_servers = client.rdservers.list()
     except AttributeError:
         mgmt_servers = client.servers.list(search_opts={'all_tenants': 1})
-    LOG.info("Found %d servers in Nova" %
+    LOG.info(_("Found %d servers in Nova") %
              len(mgmt_servers if mgmt_servers else []))
     args = {}
     if deleted is not None:
@@ -188,11 +189,11 @@ class NotificationTransformer(object):
     @staticmethod
     def _get_audit_period():
         now = datetime.datetime.now()
-        audit_start = utils.isotime(now, subsecond=True)
-        audit_end = utils.isotime(
-            now + datetime.timedelta(
+        audit_start = utils.isotime(
+            now - datetime.timedelta(
                 seconds=CONF.exists_notification_interval),
             subsecond=True)
+        audit_end = utils.isotime(now, subsecond=True)
         return audit_start, audit_end
 
     def _get_service_id(self, datastore_manager, id_map):
@@ -200,7 +201,7 @@ class NotificationTransformer(object):
             datastore_manager_id = id_map[datastore_manager]
         else:
             datastore_manager_id = cfg.UNKNOWN_SERVICE_ID
-            LOG.error("Datastore ID for Manager (%s) is not configured"
+            LOG.error(_("Datastore ID for Manager (%s) is not configured")
                       % datastore_manager)
         return datastore_manager_id
 
@@ -259,7 +260,7 @@ class NovaNotificationTransformer(NotificationTransformer):
             LOG.debug("Flavor cache hit for %s" % flavor_id)
             return self._flavor_cache[flavor_id]
         # fetch flavor resource from nova
-        LOG.info("Flavor cache miss for %s" % flavor_id)
+        LOG.info(_("Flavor cache miss for %s") % flavor_id)
         flavor = self.nova_client.flavors.get(flavor_id)
         self._flavor_cache[flavor_id] = flavor.name if flavor else 'unknown'
         return self._flavor_cache[flavor_id]
