@@ -25,6 +25,7 @@ from oslo_messaging.rpc.client import RemoteError
 from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
+from trove.common.notification import NotificationCastWrapper
 import trove.common.rpc.version as rpc_version
 from trove import rpc
 
@@ -75,8 +76,9 @@ class API(object):
     def _cast(self, method_name, version, **kwargs):
         LOG.debug("Casting %s" % method_name)
         try:
-            cctxt = self.client.prepare(version=version)
-            cctxt.cast(self.context, method_name, **kwargs)
+            with NotificationCastWrapper(self.context, 'guest'):
+                cctxt = self.client.prepare(version=version)
+                cctxt.cast(self.context, method_name, **kwargs)
         except RemoteError as r:
             LOG.exception(_("Error calling %s") % method_name)
             raise exception.GuestError(original_message=r.value)
@@ -225,7 +227,8 @@ class API(object):
     def prepare(self, memory_mb, packages, databases, users,
                 device_path='/dev/vdb', mount_point='/mnt/volume',
                 backup_info=None, config_contents=None, root_password=None,
-                overrides=None, cluster_config=None, snapshot=None):
+                overrides=None, cluster_config=None, snapshot=None,
+                modules=None):
         """Make an asynchronous call to prepare the guest
            as a database container optionally includes a backup id for restores
         """
@@ -244,7 +247,7 @@ class API(object):
             device_path=device_path, mount_point=mount_point,
             backup_info=backup_info, config_contents=config_contents,
             root_password=root_password, overrides=overrides,
-            cluster_config=cluster_config, snapshot=snapshot)
+            cluster_config=cluster_config, snapshot=snapshot, modules=modules)
 
     def _create_guest_queue(self):
         """Call to construct, start and immediately stop rpc server in order
@@ -262,6 +265,7 @@ class API(object):
         finally:
             if server is not None:
                 server.stop()
+                server.wait()
 
     def restart(self):
         """Restart the database server."""
@@ -426,3 +430,35 @@ class API(object):
         LOG.debug("Demoting instance %s to non-master.", self.id)
         self._call("demote_replication_master", AGENT_HIGH_TIMEOUT,
                    self.version_cap)
+
+    def guest_log_list(self):
+        LOG.debug("Retrieving guest log list for %s.", self.id)
+        result = self._call("guest_log_list", AGENT_HIGH_TIMEOUT,
+                            self.version_cap)
+        LOG.debug("guest_log_list returns %s", result)
+        return result
+
+    def guest_log_action(self, log_name, enable, disable, publish, discard):
+        LOG.debug("Processing guest log '%s' for %s.", log_name, self.id)
+        return self._call("guest_log_action", AGENT_HIGH_TIMEOUT,
+                          self.version_cap, log_name=log_name,
+                          enable=enable, disable=disable,
+                          publish=publish, discard=discard)
+
+    def module_list(self, include_contents):
+        LOG.debug("Querying modules on %s (contents: %s).",
+                  self.id, include_contents)
+        result = self._call("module_list", AGENT_HIGH_TIMEOUT,
+                            self.version_cap,
+                            include_contents=include_contents)
+        return result
+
+    def module_apply(self, modules):
+        LOG.debug("Applying modules to %s.", self.id)
+        return self._call("module_apply", AGENT_HIGH_TIMEOUT,
+                          self.version_cap, modules=modules)
+
+    def module_remove(self, module):
+        LOG.debug("Removing modules from %s.", self.id)
+        return self._call("module_remove", AGENT_HIGH_TIMEOUT,
+                          self.version_cap, module=module)
