@@ -19,6 +19,7 @@ from oslo_log import log as logging
 from trove.backup.state import BackupState
 from trove.common import cfg
 from trove.common.i18n import _
+from trove.common.strategies.storage import get_storage_strategy
 from trove.conductor import api as conductor_api
 from trove.guestagent.common import timeutils
 from trove.guestagent.dbaas import get_filesystem_volume_stats
@@ -26,7 +27,6 @@ from trove.guestagent.strategies.backup.base import BackupError
 from trove.guestagent.strategies.backup.base import UnknownBackupType
 from trove.guestagent.strategies.backup import get_backup_strategy
 from trove.guestagent.strategies.restore import get_restore_strategy
-from trove.guestagent.strategies.storage import get_storage_strategy
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
