@@ -33,50 +33,91 @@ DEBIAN = 'debian'
 SUSE = 'suse'
 
 
-def read_file(path, codec=IdentityCodec(), as_root=False):
+def read_file(path, codec=IdentityCodec(), as_root=False, decode=True):
     """
     Read a file into a Python data structure
     digestible by 'write_file'.
 
-    :param path             Path to the read config file.
-    :type path              string
+    :param path:            Path to the read config file.
+    :type path:             string
 
-    :param codec:           A codec used to deserialize the data.
+    :param codec:           A codec used to transform the data.
     :type codec:            StreamCodec
 
-    :returns:               A dictionary of key-value pairs.
-
     :param as_root:         Execute as root.
     :type as_root:          boolean
 
+    :param decode:          Should the codec decode the data.
+    :type decode:           boolean
+
+    :returns:               A dictionary of key-value pairs.
+
     :raises:                :class:`UnprocessableEntity` if file doesn't exist.
     :raises:                :class:`UnprocessableEntity` if codec not given.
     """
-    if path and os.path.exists(path):
+    if path and exists(path, is_directory=False, as_root=as_root):
         if as_root:
-            return _read_file_as_root(path, codec)
-        with open(path, 'r') as fp:
-            return codec.deserialize(fp.read())
+            return _read_file_as_root(path, codec, decode=decode)
+
+        with open(path, 'rb') as fp:
+            if decode:
+                return codec.deserialize(fp.read())
+            return codec.serialize(fp.read())
 
     raise exception.UnprocessableEntity(_("File does not exist: %s") % path)
 
 
-def _read_file_as_root(path, codec):
+def exists(path, is_directory=False, as_root=False):
+    """Check a given path exists.
+
+    :param path                Path to be checked.
+    :type path                 string
+
+    :param is_directory:       Check that the path exists and is a directory.
+                               Check for a regular file otherwise.
+    :type is_directory:        boolean
+
+    :param as_root:            Execute as root.
+    :type as_root:             boolean
+    """
+
+    found = (not is_directory and os.path.isfile(path) or
+             (is_directory and os.path.isdir(path)))
+
+    # Only check as root if we can't see it as the regular user, since
+    # this is more expensive
+    if not found and as_root:
+        test_flag = '-d' if is_directory else '-f'
+        cmd = 'test %s %s && echo 1 || echo 0' % (test_flag, path)
+        stdout, _ = utils.execute_with_timeout(
+            cmd, shell=True, check_exit_code=False,
+            run_as_root=True, root_helper='sudo')
+        found = bool(int(stdout))
+
+    return found
+
+
+def _read_file_as_root(path, codec, decode=True):
     """Read a file as root.
 
     :param path                Path to the written file.
     :type path                 string
 
-    :param codec:              A codec used to serialize the data.
+    :param codec:              A codec used to transform the data.
     :type codec:               StreamCodec
+
+    :param decode:             Should the codec decode the data.
+    :type decode:              boolean
     """
     with tempfile.NamedTemporaryFile() as fp:
         copy(path, fp.name, force=True, as_root=True)
         chmod(fp.name, FileMode.ADD_READ_ALL(), as_root=True)
-        return codec.deserialize(fp.read())
+        if decode:
+            return codec.deserialize(fp.read())
+        return codec.serialize(fp.read())
 
 
-def write_file(path, data, codec=IdentityCodec(), as_root=False):
+def write_file(path, data, codec=IdentityCodec(), as_root=False, encode=True):
     """Write data into file using a given codec.
     Overwrite any existing contents.
     The written file can be read back into its original
@@ -88,25 +129,31 @@ def write_file(path, data, codec=IdentityCodec(), as_root=False):
     :param data:               An object representing the file contents.
     :type data:                object
 
-    :param codec:              A codec used to serialize the data.
+    :param codec:              A codec used to transform the data.
     :type codec:               StreamCodec
 
-    :param codec:              Execute as root.
-    :type codec:               boolean
+    :param as_root:            Execute as root.
+    :type as_root:             boolean
+
+    :param encode:             Should the codec encode the data.
+    :type encode:              boolean
 
     :raises:                   :class:`UnprocessableEntity` if path not given.
     """
     if path:
         if as_root:
-            _write_file_as_root(path, data, codec)
+            _write_file_as_root(path, data, codec, encode=encode)
         else:
-            with open(path, 'w', 0) as fp:
-                fp.write(codec.serialize(data))
+            with open(path, 'wb', 0) as fp:
+                if encode:
+                    fp.write(codec.serialize(data))
+                else:
+                    fp.write(codec.deserialize(data))
     else:
         raise exception.UnprocessableEntity(_("Invalid path: %s") % path)
 
 
-def _write_file_as_root(path, data, codec):
+def _write_file_as_root(path, data, codec, encode=True):
     """Write a file as root. Overwrite any existing contents.
 
     :param path                Path to the written file.
@@ -115,13 +162,19 @@ def _write_file_as_root(path, data, codec):
     :param data:               An object representing the file contents.
     :type data:                StreamCodec
 
-    :param codec:              A codec used to serialize the data.
+    :param codec:              A codec used to transform the data.
     :type codec:               StreamCodec
+
+    :param encode:             Should the codec encode the data.
+    :type encode:              boolean
     """
     # The files gets removed automatically once the managing object goes
     # out of scope.
-    with tempfile.NamedTemporaryFile('w', 0, delete=False) as fp:
-        fp.write(codec.serialize(data))
+    with tempfile.NamedTemporaryFile('wb', 0, delete=False) as fp:
+        if encode:
+            fp.write(codec.serialize(data))
+        else:
+            fp.write(codec.deserialize(data))
         fp.close()  # Release the resource before proceeding.
         copy(fp.name, path, force=True, as_root=True)
 
@@ -156,9 +209,13 @@ class FileMode(object):
     """
 
     @classmethod
-    def SET_FULL(cls):
+    def SET_ALL_RWX(cls):
         return cls(reset=[stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO])  # =0777
 
+    @classmethod
+    def SET_FULL(cls):
+        return cls.SET_ALL_RWX()
+
     @classmethod
     def SET_GRP_RW_OTH_R(cls):
         return cls(reset=[stat.S_IRGRP | stat.S_IWGRP | stat.S_IROTH])  # =0064
@@ -172,13 +229,37 @@ class FileMode(object):
         return cls(reset=[stat.S_IRUSR | stat.S_IWUSR])  # =0600
 
     @classmethod
-    def ADD_READ_ALL(cls):
+    def ADD_ALL_R(cls):
         return cls(add=[stat.S_IRUSR | stat.S_IRGRP | stat.S_IROTH])  # +0444
 
+    @classmethod
+    def ADD_READ_ALL(cls):
+        return cls.ADD_ALL_R()
+
+    @classmethod
+    def ADD_USR_RW_GRP_RW(cls):
+        return cls(add=[stat.S_IRUSR | stat.S_IWUSR |
+                        stat.S_IRGRP | stat.S_IWGRP])  # +0660
+
+    @classmethod
+    def ADD_USR_RW_GRP_RW_OTH_R(cls):
+        return cls(add=[stat.S_IRUSR | stat.S_IWUSR |
+                        stat.S_IRGRP | stat.S_IWGRP |
+                        stat.S_IROTH])  # +0664
+
     @classmethod
     def ADD_GRP_RW(cls):
         return cls(add=[stat.S_IRGRP | stat.S_IWGRP])  # +0060
 
+    @classmethod
+    def ADD_GRP_RX(cls):
+        return cls(add=[stat.S_IRGRP | stat.S_IXGRP])  # +0050
+
+    @classmethod
+    def ADD_GRP_RX_OTH_RX(cls):
+        return cls(add=[stat.S_IRGRP | stat.S_IXGRP |
+                        stat.S_IROTH | stat.S_IXOTH])  # +0055
+
     def __init__(self, reset=None, add=None, remove=None):
         self._reset = list(reset) if reset is not None else []
         self._add = list(add) if add is not None else []
@@ -248,6 +329,7 @@ def file_discovery(file_candidates):
     for file in file_candidates:
         if os.path.isfile(file):
             return file
+    return ''
 
 
 def start_service(service_candidates):
@@ -470,6 +552,42 @@ def chmod(path, mode, recursive=True, force=False, **kwargs):
             _("Cannot change mode of a blank file."))
 
 
+def change_user_group(user, group, append=True, add_group=True, **kwargs):
+    """Adds a user to groups by using the usermod linux command with -a and
+    -G options.
+
+    seealso:: _execute_shell_cmd for valid optional keyword arguments.
+
+    :param user:            Username.
+    :type user:             string
+
+    :param group:           Group names.
+    :type group:            comma separated string
+
+    :param  append:         Adds user to a group.
+    :type append:           boolean
+
+    :param add_group:       Lists the groups that the user is a member of.
+                            While adding a new groups to an existing user
+                            with '-G' option alone, will remove all existing
+                            groups that user belongs. Therefore, always add
+                            the '-a' (append) with '-G' option to add or
+                            append new groups.
+    :type add_group:        boolean
+
+    :raises:                :class:`UnprocessableEntity` if user or group not
+                            given.
+    """
+
+    if not user:
+        raise exception.UnprocessableEntity(_("Missing user."))
+    elif not group:
+        raise exception.UnprocessableEntity(_("Missing group."))
+
+    options = (('a', append), ('G', add_group))
+    _execute_shell_cmd('usermod', options, group, user, **kwargs)
+
+
 def _build_shell_chmod_mode(mode):
     """
     Build a shell representation of given mode.
@@ -598,6 +716,42 @@ def get_bytes_free_on_fs(path):
     return v.f_bsize * v.f_bavail
 
 
+def list_files_in_directory(root_dir, recursive=False, pattern=None,
+                            include_dirs=False, as_root=False):
+    """
+    Return absolute paths to all files in a given root directory.
+
+    :param root_dir            Path to the root directory.
+    :type root_dir             string
+
+    :param recursive           Also descend into sub-directories if True.
+    :type recursive            boolean
+
+    :param pattern             Return only names matching the pattern.
+    :type pattern              string
+
+    :param include_dirs        Include paths to individual sub-directories.
+    :type include_dirs         boolean
+    """
+    if as_root:
+        cmd_args = [root_dir, '-noleaf']
+        if not recursive:
+            cmd_args.extend(['-maxdepth', '0'])
+        if not include_dirs:
+            cmd_args.extend(['-type', 'f'])
+        if pattern:
+            cmd_args.extend(['-regextype', 'posix-extended',
+                             '-regex', os.path.join('.*', pattern) + '$'])
+        files = _execute_shell_cmd('find', [], *cmd_args, as_root=True)
+        return {fp for fp in files.splitlines()}
+
+    return {os.path.abspath(os.path.join(root, name))
+            for (root, dirs, files) in os.walk(root_dir, topdown=True)
+            if recursive or (root == root_dir)
+            for name in (files + (dirs if include_dirs else []))
+            if not pattern or re.match(pattern, name)}
+
+
 def _execute_shell_cmd(cmd, options, *args, **kwargs):
     """Execute a given shell command passing it
     given options (flags) and arguments.
@@ -627,7 +781,8 @@ def _execute_shell_cmd(cmd, options, *args, **kwargs):
 
     cmd_flags = _build_command_options(options)
     cmd_args = cmd_flags + list(args)
-    utils.execute_with_timeout(cmd, *cmd_args, **exec_args)
+    stdout, stderr = utils.execute_with_timeout(cmd, *cmd_args, **exec_args)
+    return stdout
 
 
 def _build_command_options(options):
@@ -637,23 +792,3 @@ def _build_command_options(options):
     """
 
     return ['-' + item[0] for item in options if item[1]]
-
-
-def list_files_in_directory(root_dir, recursive=False, pattern=None):
-    """
-    Return absolute paths to all files in a given root directory.
-
-    :param root_dir            Path to the root directory.
-    :type root_dir             string
-
-    :param recursive           Also probe subdirectories if True.
-    :type recursive            boolean
-
-    :param pattern             Return only files matching the pattern.
-    :type pattern              string
-    """
-    return {os.path.abspath(os.path.join(root, name))
-            for (root, _, files) in os.walk(root_dir, topdown=True)
-            if recursive or (root == root_dir)
-            for name in files
-            if not pattern or re.match(pattern, name)}
