@@ -21,7 +21,6 @@ import pymongo
 
 from trove.common import cfg
 from trove.common import exception
-from trove.common.exception import ProcessExecutionError
 from trove.common.i18n import _
 from trove.common import instance as ds_instance
 from trove.common import pagination
@@ -39,39 +38,24 @@ from trove.guestagent.db import models
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
 CONFIG_FILE = operating_system.file_discovery(system.CONFIG_CANDIDATES)
+MANAGER = CONF.datastore_manager if CONF.datastore_manager else 'mongodb'
 
 # Configuration group for clustering-related settings.
 CNF_CLUSTER = 'clustering'
 
 MONGODB_PORT = CONF.mongodb.mongodb_port
 CONFIGSVR_PORT = CONF.mongodb.configsvr_port
-IGNORED_DBS = CONF.mongodb.ignore_dbs
-IGNORED_USERS = CONF.mongodb.ignore_users
 
 
 class MongoDBApp(object):
     """Prepares DBaaS on a Guest container."""
 
-    @classmethod
-    def _init_overrides_dir(cls):
-        """Initialize a directory for configuration overrides.
-        """
-        revision_dir = guestagent_utils.build_file_path(
-            os.path.dirname(CONFIG_FILE),
-            ConfigurationManager.DEFAULT_STRATEGY_OVERRIDES_SUB_DIR)
-
-        if not os.path.exists(revision_dir):
-            operating_system.create_directory(
-                revision_dir,
-                user=system.MONGO_USER, group=system.MONGO_USER,
-                force=True, as_root=True)
-
-        return revision_dir
-
     def __init__(self):
         self.state_change_wait_time = CONF.state_change_wait_time
 
-        revision_dir = self._init_overrides_dir()
+        revision_dir = guestagent_utils.build_file_path(
+            os.path.dirname(CONFIG_FILE),
+            ConfigurationManager.DEFAULT_STRATEGY_OVERRIDES_SUB_DIR)
         self.configuration_manager = ConfigurationManager(
             CONFIG_FILE, system.MONGO_USER, system.MONGO_USER,
             SafeYamlCodec(default_flow_style=False),
@@ -90,99 +74,24 @@ class MongoDBApp(object):
             system.PACKAGER.pkg_install(packages, {}, system.TIME_OUT)
         LOG.info(_("Finished installing MongoDB server."))
 
-    def _get_service(self):
+    def _get_service_candidates(self):
         if self.is_query_router:
-            return (operating_system.
-                    service_discovery(system.MONGOS_SERVICE_CANDIDATES))
-        else:
-            return (operating_system.
-                    service_discovery(system.MONGOD_SERVICE_CANDIDATES))
-
-    def _enable_db_on_boot(self):
-        LOG.info(_("Enabling MongoDB on boot."))
-        try:
-            mongo_service = self._get_service()
-            utils.execute_with_timeout(mongo_service['cmd_enable'],
-                                       shell=True)
-        except KeyError:
-            raise RuntimeError(_("MongoDB service is not discovered."))
-
-    def _disable_db_on_boot(self):
-        LOG.info(_("Disabling MongoDB on boot."))
-        try:
-            mongo_service = self._get_service()
-            utils.execute_with_timeout(mongo_service['cmd_disable'],
-                                       shell=True)
-        except KeyError:
-            raise RuntimeError("MongoDB service is not discovered.")
+            return system.MONGOS_SERVICE_CANDIDATES
+        return system.MONGOD_SERVICE_CANDIDATES
 
     def stop_db(self, update_db=False, do_not_start_on_reboot=False):
-        LOG.info(_("Stopping MongoDB."))
-        if do_not_start_on_reboot:
-            self._disable_db_on_boot()
-
-        try:
-            mongo_service = self._get_service()
-            # TODO(ramashri) see if hardcoded values can be removed
-            utils.execute_with_timeout(mongo_service['cmd_stop'],
-                                       shell=True, timeout=100)
-        except KeyError:
-            raise RuntimeError(_("MongoDB service is not discovered."))
-
-        if not self.status.wait_for_real_status_to_change_to(
-                ds_instance.ServiceStatuses.SHUTDOWN,
-                self.state_change_wait_time, update_db):
-            LOG.error(_("Could not stop MongoDB."))
-            self.status.end_install_or_restart()
-            raise RuntimeError(_("Could not stop MongoDB"))
+        self.status.stop_db_service(
+            self._get_service_candidates(), self.state_change_wait_time,
+            disable_on_boot=do_not_start_on_reboot, update_db=update_db)
 
     def restart(self):
-        LOG.info(_("Restarting MongoDB."))
-        try:
-            self.status.begin_restart()
-            self.stop_db()
-            self.start_db()
-        finally:
-            self.status.end_install_or_restart()
+        self.status.restart_db_service(
+            self._get_service_candidates(), self.state_change_wait_time)
 
     def start_db(self, update_db=False):
-        LOG.info(_("Starting MongoDB."))
-
-        self._enable_db_on_boot()
-
-        try:
-            mongo_service = self._get_service()
-            utils.execute_with_timeout(mongo_service['cmd_start'],
-                                       shell=True)
-        except ProcessExecutionError:
-            pass
-        except KeyError:
-            raise RuntimeError("MongoDB service is not discovered.")
-        self.wait_for_start(update_db=update_db)
-
-    def wait_for_start(self, update_db=False):
-        LOG.debug('Waiting for MongoDB to start.')
-        if not self.status.wait_for_real_status_to_change_to(
-                ds_instance.ServiceStatuses.RUNNING,
-                self.state_change_wait_time, update_db):
-            LOG.error(_("Start up of MongoDB failed."))
-            # If it won't start, but won't die either, kill it by hand so we
-            # don't let a rouge process wander around.
-            try:
-                out, err = utils.execute_with_timeout(
-                    system.FIND_PID, shell=True)
-                pid = "".join(out.split(" ")[1:2])
-                utils.execute_with_timeout(
-                    system.MONGODB_KILL % pid, shell=True)
-            except exception.ProcessExecutionError:
-                LOG.exception(_("Error killing MongoDB start command."))
-                # There's nothing more we can do...
-            self.status.end_install_or_restart()
-            raise RuntimeError("Could not start MongoDB.")
-        LOG.debug('MongoDB started successfully.')
-
-    def complete_install_or_restart(self):
-        self.status.end_install_or_restart()
+        self.status.start_db_service(
+            self._get_service_candidates(), self.state_change_wait_time,
+            enable_on_boot=True, update_db=update_db)
 
     def update_overrides(self, context, overrides, remove=False):
         if overrides:
@@ -205,11 +114,6 @@ class MongoDBApp(object):
             None, mount_point=system.MONGODB_MOUNT_POINT)
         self.start_db(True)
 
-    def reset_configuration(self, configuration):
-        LOG.info(_("Resetting configuration."))
-        config_contents = configuration['config_contents']
-        self.configuration_manager.save_configuration(config_contents)
-
     def apply_initial_guestagent_configuration(
             self, cluster_config, mount_point=None):
         LOG.debug("Applying initial configuration.")
@@ -219,8 +123,6 @@ class MongoDBApp(object):
         # See: https://jira.mongodb.org/browse/SERVER-20075
         self._initialize_writable_run_dir()
 
-        # todo mvandijk: enable authorization.
-        # 'security.authorization': True
         self.configuration_manager.apply_system_override(
             {'processManagement.fork': False,
              'processManagement.pidFilePath': system.MONGO_PID_FILE,
@@ -271,16 +173,6 @@ class MongoDBApp(object):
         LOG.info(_("Configuring instance as a cluster query router."))
         self.is_query_router = True
 
-        # Write the 'mongos' upstart script.
-        # FIXME(pmalik): The control script should really be written in the
-        # elements.
-        # The guestagent will choose the right daemon ('mongod' or 'mongos')
-        # based on the 'cluster_config' values.
-        upstart_contents = (system.MONGOS_UPSTART_CONTENTS.
-                            format(config_file_placeholder=CONFIG_FILE))
-        operating_system.write_file(system.MONGOS_UPSTART, upstart_contents,
-                                    as_root=True)
-
         # FIXME(pmalik): We should really have a separate configuration
         # template for the 'mongos' process.
         # Remove all storage configurations from the template.
@@ -321,14 +213,15 @@ class MongoDBApp(object):
 
     def _configure_cluster_security(self, key_value):
         """Force cluster key-file-based authentication.
+
+        This will enabled RBAC.
         """
         # Store the cluster member authentication key.
         self.store_key(key_value)
 
-        # TODO(mvandijk): enable cluster security once Trove features are in
-        # self.configuration_manager.apply_system_override(
-        #     {'security.clusterAuthMode': 'keyFile',
-        #      'security.keyFile': self.get_key_file()}, CNF_CLUSTER)
+        self.configuration_manager.apply_system_override(
+            {'security.clusterAuthMode': 'keyFile',
+             'security.keyFile': self.get_key_file()}, CNF_CLUSTER)
 
     def _configure_network(self, port=None):
         """Make the service accessible at a given (or default if not) port.
@@ -480,7 +373,7 @@ class MongoDBApp(object):
         # the driver engine is already cached, but we need to change it it
         with MongoDBClient(None, host='localhost',
                            port=MONGODB_PORT) as client:
-            MongoDBAdmin().create_user(user, client=client)
+            MongoDBAdmin().create_validated_user(user, client=client)
         # now revert to the normal engine
         self.status.set_host(host=netutils.get_my_ipv4(),
                              port=MONGODB_PORT)
@@ -490,12 +383,15 @@ class MongoDBApp(object):
         """Create the Trove admin user.
 
         The service should not be running at this point.
+        This will enable role-based access control (RBAC) by default.
         """
         if self.status.is_running:
             raise RuntimeError(_("Cannot secure the instance. "
                                  "The service is still running."))
 
         try:
+            self.configuration_manager.apply_system_override(
+                {'security.authorization': 'enabled'})
             self._set_localhost_auth_bypass(True)
             self.start_db(update_db=False)
             password = utils.generate_random_password()
@@ -561,6 +457,11 @@ class MongoDBAppStatus(service.BaseDbStatus):
 
         return ds_instance.ServiceStatuses.SHUTDOWN
 
+    def cleanup_stalled_db_services(self):
+        out, err = utils.execute_with_timeout(system.FIND_PID, shell=True)
+        pid = "".join(out.split(" ")[1:2])
+        utils.execute_with_timeout(system.MONGODB_KILL % pid, shell=True)
+
 
 class MongoDBAdmin(object):
     """Handles administrative tasks on MongoDB."""
@@ -579,6 +480,12 @@ class MongoDBAdmin(object):
             type(self).admin_user = user
         return type(self).admin_user
 
+    def _is_modifiable_user(self, name):
+        if ((name in cfg.get_ignored_users(manager=MANAGER)) or
+                name == system.MONGO_ADMIN_NAME):
+            return False
+        return True
+
     @property
     def cmd_admin_auth_params(self):
         """Returns a list of strings that constitute MongoDB command line
@@ -595,8 +502,11 @@ class MongoDBAdmin(object):
             user.username, password=user.password, roles=user.roles
         )
 
-    def create_user(self, user, client=None):
-        """Creates a user on their database."""
+    def create_validated_user(self, user, client=None):
+        """Creates a user on their database. The caller should ensure that
+        this action is valid.
+        :param user:   a MongoDBUser object
+        """
         LOG.debug('Creating user %s on database %s with roles %s.'
                   % (user.username, user.database.name, str(user.roles)))
 
@@ -610,36 +520,69 @@ class MongoDBAdmin(object):
                 self._create_user_with_client(user, admin_client)
 
     def create_users(self, users):
-        """Create the given user(s)."""
+        """Create the given user(s).
+        :param users:   list of serialized user objects
+        """
         with MongoDBClient(self._admin_user()) as client:
-            for user in users:
-                self.create_user(models.MongoDBUser.deserialize_user(user),
-                                 client)
+            for item in users:
+                user = models.MongoDBUser.deserialize_user(item)
+                if not self._is_modifiable_user(user.name):
+                    LOG.warning('Skipping creation of user with reserved '
+                                'name %(user)s' % {'user': user.name})
+                elif self._get_user_record(user.name, client=client):
+                    LOG.warning('Skipping creation of user with pre-existing '
+                                'name %(user)s' % {'user': user.name})
+                else:
+                    self.create_validated_user(user, client=client)
+
+    def delete_validated_user(self, user):
+        """Deletes a user from their database. The caller should ensure that
+        this action is valid.
+        :param user:   a MongoDBUser object
+        """
+        LOG.debug('Deleting user %s from database %s.'
+                  % (user.username, user.database.name))
+        with MongoDBClient(self._admin_user()) as admin_client:
+            admin_client[user.database.name].remove_user(user.username)
 
     def delete_user(self, user):
-        """Delete the given user."""
+        """Delete the given user.
+        :param user:   a serialized user object
+        """
         user = models.MongoDBUser.deserialize_user(user)
-        username = user.username
-        db_name = user.database.name
-        LOG.debug('Deleting user %s from database %s.' % (username, db_name))
-        with MongoDBClient(self._admin_user()) as admin_client:
-            admin_client[db_name].remove_user(username)
+        if not self._is_modifiable_user(user.name):
+            raise exception.BadRequest(_(
+                'Cannot delete user with reserved name %(user)s')
+                % {'user': user.name})
+        else:
+            self.delete_validated_user(user)
 
-    def _get_user_record(self, name):
+    def _get_user_record(self, name, client=None):
         """Get the user's record."""
         user = models.MongoDBUser(name)
-        with MongoDBClient(self._admin_user()) as admin_client:
-            user_info = admin_client.admin.system.users.find_one(
+        if not self._is_modifiable_user(user.name):
+            LOG.warning('Skipping retrieval of user with reserved '
+                        'name %(user)s' % {'user': user.name})
+            return None
+        if client:
+            user_info = client.admin.system.users.find_one(
                 {'user': user.username, 'db': user.database.name})
-            if not user_info:
-                return None
-            user.roles = user_info['roles']
+        else:
+            with MongoDBClient(self._admin_user()) as admin_client:
+                user_info = admin_client.admin.system.users.find_one(
+                    {'user': user.username, 'db': user.database.name})
+        if not user_info:
+            return None
+        user.roles = user_info['roles']
         return user
 
     def get_user(self, name):
         """Get information for the given user."""
         LOG.debug('Getting user %s.' % name)
-        return self._get_user_record(name).serialize()
+        user = self._get_user_record(name)
+        if not user:
+            return None
+        return user.serialize()
 
     def list_users(self, limit=None, marker=None, include_marker=False):
         """Get a list of all users."""
@@ -648,12 +591,41 @@ class MongoDBAdmin(object):
             for user_info in admin_client.admin.system.users.find():
                 user = models.MongoDBUser(name=user_info['_id'])
                 user.roles = user_info['roles']
-                if user.name not in IGNORED_USERS:
+                if self._is_modifiable_user(user.name):
                     users.append(user.serialize())
         LOG.debug('users = ' + str(users))
         return pagination.paginate_list(users, limit, marker,
                                         include_marker)
 
+    def change_passwords(self, users):
+        with MongoDBClient(self._admin_user()) as admin_client:
+            for item in users:
+                user = models.MongoDBUser.deserialize_user(item)
+                if not self._is_modifiable_user(user.name):
+                    LOG.warning('Skipping password change for user with '
+                                'reserved name %(user)s.'
+                                % {'user': user.name})
+                    return None
+                LOG.debug('Changing password for user %(user)s'
+                          % {'user': user.name})
+                self._create_user_with_client(user, admin_client)
+
+    def update_attributes(self, name, user_attrs):
+        """Update user attributes."""
+        user = self._get_user_record(name)
+        if not user:
+            raise exception.BadRequest(_(
+                'Cannot update attributes for user %(user)s as it either does '
+                'not exist or is a reserved user.') % {'user': name})
+        password = user_attrs.get('password')
+        if password:
+            user.password = password
+            self.change_passwords([user.serialize()])
+        if user_attrs.get('name'):
+            LOG.warning('Changing user name is not supported.')
+        if user_attrs.get('host'):
+            LOG.warning('Changing user host is not supported.')
+
     def enable_root(self, password=None):
         """Create a user 'root' with role 'root'."""
         if not password:
@@ -661,7 +633,7 @@ class MongoDBAdmin(object):
             password = utils.generate_random_password()
         root_user = models.MongoDBUser(name='admin.root', password=password)
         root_user.roles = {'db': 'admin', 'role': 'root'}
-        self.create_user(root_user)
+        self.create_validated_user(root_user)
         return root_user.serialize()
 
     def is_root_enabled(self):
@@ -680,6 +652,10 @@ class MongoDBAdmin(object):
     def grant_access(self, username, databases):
         """Adds the RW role to the user for each specified database."""
         user = self._get_user_record(username)
+        if not user:
+            raise exception.BadRequest(_(
+                'Cannot grant access for reserved or non-existant user '
+                '%(user)s') % {'user': username})
         for db_name in databases:
             # verify the database name
             models.MongoDBSchema(db_name)
@@ -697,6 +673,10 @@ class MongoDBAdmin(object):
     def revoke_access(self, username, database):
         """Removes the RW role from the user for the specified database."""
         user = self._get_user_record(username)
+        if not user:
+            raise exception.BadRequest(_(
+                'Cannot revoke access for reserved or non-existant user '
+                '%(user)s') % {'user': username})
         # verify the database name
         models.MongoDBSchema(database)
         role = {'db': database, 'role': 'readWrite'}
@@ -710,6 +690,10 @@ class MongoDBAdmin(object):
         """Returns a list of all databases for which the user has the RW role.
         """
         user = self._get_user_record(username)
+        if not user:
+            raise exception.BadRequest(_(
+                'Cannot list access for reserved or non-existant user '
+                '%(user)s') % {'user': username})
         return user.databases
 
     def create_database(self, databases):
@@ -740,7 +724,7 @@ class MongoDBAdmin(object):
     def list_databases(self, limit=None, marker=None, include_marker=False):
         """Lists the databases."""
         db_names = self.list_database_names()
-        for hidden in IGNORED_DBS:
+        for hidden in cfg.get_ignored_dbs(manager=MANAGER):
             if hidden in db_names:
                 db_names.remove(hidden)
         databases = [models.MongoDBSchema(db_name).serialize()
