@@ -34,7 +34,7 @@ from trove.guestagent.datastore import service
 from trove.guestagent import pkg
 
 LOG = logging.getLogger(__name__)
-TIME_OUT = 1200  # FIXME(pmalik): should probably use config timeout
+TIME_OUT = 1200
 CONF = cfg.CONF
 CLUSTER_CFG = 'clustering'
 packager = pkg.Package()
@@ -65,6 +65,12 @@ class RedisAppStatus(service.BaseDbStatus):
 
         return rd_instance.ServiceStatuses.CRASHED
 
+    def cleanup_stalled_db_services(self):
+        utils.execute_with_timeout('pkill', '-9',
+                                   'redis-server',
+                                   run_as_root=True,
+                                   root_helper='sudo')
+
 
 class RedisApp(object):
     """
@@ -72,22 +78,6 @@ class RedisApp(object):
     on a trove instance.
     """
 
-    @classmethod
-    def _init_overrides_dir(cls):
-        """Initialize a directory for configuration overrides.
-        """
-        revision_dir = guestagent_utils.build_file_path(
-            os.path.dirname(system.REDIS_CONFIG),
-            ConfigurationManager.DEFAULT_STRATEGY_OVERRIDES_SUB_DIR)
-
-        if not os.path.exists(revision_dir):
-            operating_system.create_directory(
-                revision_dir,
-                user=system.REDIS_OWNER, group=system.REDIS_OWNER,
-                force=True, as_root=True)
-
-        return revision_dir
-
     def __init__(self, state_change_wait_time=None):
         """
         Sets default status and state_change_wait_time
@@ -97,7 +87,9 @@ class RedisApp(object):
         else:
             self.state_change_wait_time = CONF.state_change_wait_time
 
-        revision_dir = self._init_overrides_dir()
+        revision_dir = guestagent_utils.build_file_path(
+            os.path.dirname(system.REDIS_CONFIG),
+            ConfigurationManager.DEFAULT_STRATEGY_OVERRIDES_SUB_DIR)
         config_value_mappings = {'yes': True, 'no': False, "''": None}
         self._value_converter = StringConverter(config_value_mappings)
         self.configuration_manager = ConfigurationManager(
@@ -128,13 +120,6 @@ class RedisApp(object):
             self._install_redis(packages)
         LOG.info(_('Redis installed completely.'))
 
-    def complete_install_or_restart(self):
-        """
-        finalize status updates for install or restart.
-        """
-        LOG.debug("Complete install or restart called.")
-        self.status.end_install_or_restart()
-
     def _install_redis(self, packages):
         """
         Install the redis server.
@@ -145,49 +130,17 @@ class RedisApp(object):
         operating_system.create_directory(system.REDIS_CONF_DIR, as_root=True)
         pkg_opts = {}
         packager.pkg_install(packages, pkg_opts, TIME_OUT)
-        self.start_redis()
+        self.start_db()
         LOG.debug('Finished installing redis server.')
 
-    def _enable_redis_on_boot(self):
-        """
-        Enables redis on boot.
-        """
-        LOG.info(_('Enabling Redis on boot.'))
-        operating_system.enable_service_on_boot(system.SERVICE_CANDIDATES)
-
-    def _disable_redis_on_boot(self):
-        """
-        Disables redis on boot.
-        """
-        LOG.info(_("Disabling Redis on boot."))
-        operating_system.disable_service_on_boot(system.SERVICE_CANDIDATES)
-
     def stop_db(self, update_db=False, do_not_start_on_reboot=False):
-        """
-        Stops the redis application on the trove instance.
-        """
-        LOG.info(_('Stopping redis.'))
-        if do_not_start_on_reboot:
-            self._disable_redis_on_boot()
-
-        operating_system.stop_service(system.SERVICE_CANDIDATES)
-        if not self.status.wait_for_real_status_to_change_to(
-                rd_instance.ServiceStatuses.SHUTDOWN,
-                self.state_change_wait_time, update_db):
-            LOG.error(_('Could not stop Redis.'))
-            self.status.end_install_or_restart()
+        self.status.stop_db_service(
+            system.SERVICE_CANDIDATES, self.state_change_wait_time,
+            disable_on_boot=do_not_start_on_reboot, update_db=update_db)
 
     def restart(self):
-        """
-        Restarts the redis daemon.
-        """
-        LOG.debug("Restarting Redis daemon.")
-        try:
-            self.status.begin_restart()
-            self.stop_db()
-            self.start_redis()
-        finally:
-            self.status.end_install_or_restart()
+        self.status.restart_db_service(
+            system.SERVICE_CANDIDATES, self.state_change_wait_time)
 
     def update_overrides(self, context, overrides, remove=False):
         if overrides:
@@ -246,32 +199,12 @@ class RedisApp(object):
         # The configuration template has to be updated with
         # guestagent-controlled settings.
         self.apply_initial_guestagent_configuration()
-        self.start_redis(True)
+        self.start_db(True)
 
-    def reset_configuration(self, configuration):
-        LOG.info(_("Resetting configuration."))
-        config_contents = configuration['config_contents']
-        self.configuration_manager.save_configuration(config_contents)
-
-    def start_redis(self, update_db=False):
-        """
-        Start the redis daemon.
-        """
-        LOG.info(_("Starting redis."))
-        self._enable_redis_on_boot()
-        operating_system.start_service(system.SERVICE_CANDIDATES)
-        if not self.status.wait_for_real_status_to_change_to(
-                rd_instance.ServiceStatuses.RUNNING,
-                self.state_change_wait_time, update_db):
-            LOG.error(_("Start up of redis failed."))
-            try:
-                utils.execute_with_timeout('pkill', '-9',
-                                           'redis-server',
-                                           run_as_root=True,
-                                           root_helper='sudo')
-            except exception.ProcessExecutionError:
-                LOG.exception(_('Error killing stalled redis start command.'))
-            self.status.end_install_or_restart()
+    def start_db(self, update_db=False):
+        self.status.start_db_service(
+            system.SERVICE_CANDIDATES, self.state_change_wait_time,
+            enable_on_boot=True, update_db=update_db)
 
     def apply_initial_guestagent_configuration(self):
         """Update guestagent-controlled configuration properties.
@@ -488,7 +421,29 @@ class RedisAdmin(object):
         return self.__client.info(section=section)
 
     def persist_data(self):
-        return self.__client.save()
+        save_cmd = 'SAVE'
+        last_save = self.__client.lastsave()
+        LOG.debug("Starting Redis data persist")
+        if self.__client.bgsave():
+            save_cmd = 'BGSAVE'
+
+            def _timestamp_changed():
+                return last_save != self.__client.lastsave()
+
+            try:
+                utils.poll_until(_timestamp_changed, sleep_time=2,
+                                 time_out=TIME_OUT)
+            except exception.PollTimeOut:
+                raise RuntimeError(_("Timeout occurred waiting for Redis "
+                                   "persist (%s) to complete.") % save_cmd)
+
+        # If the background save fails for any reason, try doing a foreground
+        # one.  This blocks client connections, so we don't want it to be
+        # the default.
+        elif not self.__client.save():
+            raise exception.BackupCreationError(_("Could not persist "
+                                                "Redis data (%s)") % save_cmd)
+        LOG.debug("Redis data persist (%s) completed" % save_cmd)
 
     def set_master(self, host=None, port=None):
         self.__client.slaveof(host, port)
