@@ -22,11 +22,13 @@ import netaddr
 from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
+from trove.common import utils
 
 CONF = cfg.CONF
 
 
 class Base(object):
+
     def serialize(self):
         return self.__dict__
 
@@ -156,11 +158,82 @@ class MongoDBSchema(DatastoreSchema):
         return ['_name']
 
 
+class CassandraSchema(DatastoreSchema):
+    """Represents a Cassandra schema and its associated properties.
+
+    Keyspace names are 32 or fewer alpha-numeric characters and underscores,
+    the first of which is an alpha character.
+    """
+
+    def __init__(self, name=None, deserializing=False):
+        super(CassandraSchema, self).__init__()
+
+        if not (bool(deserializing) != bool(name)):
+            raise ValueError(_("Bad args. name: %(name)s, "
+                               "deserializing %(deser)s.")
+                             % ({'name': bool(name),
+                                 'deser': bool(deserializing)}))
+        if not deserializing:
+            self.name = name
+
+    @property
+    def _max_schema_name_length(self):
+        return 32
+
+    def _is_valid_schema_name(self, value):
+        return True
+
+    @classmethod
+    def _dict_requirements(cls):
+        return ['_name']
+
+
+class CouchDBSchema(DatastoreSchema):
+    '''Represents the CouchDB schema and its associated properties.
+
+    The database name must consist of one or more of the following characters
+    and the name must begin with a lowercase letter.
+
+    - Lowercase characters (a-z)
+    - Digits (0-9)
+    - Any of the characters _, $, (, ), +, -, and /
+    '''
+
+    name_regex = re.compile(r'^[a-z][a-z0-9_$()+/-]*$')
+
+    def __init__(self, name=None, deserializing=False):
+        super(CouchDBSchema, self).__init__()
+        self._ignore_dbs = cfg.get_ignored_dbs()
+        # need one or the other, not both, not none (!= ~ XOR)
+        if not (bool(deserializing) != bool(name)):
+            raise ValueError(_("Bad args. name: %(name)s, "
+                               "deserializing %(deser)s.")
+                             % ({'name': bool(name),
+                                 'deser': bool(deserializing)}))
+        if not deserializing:
+            self.name = name
+
+    @property
+    def _max_schema_name_length(self):
+        return None
+
+    def _is_valid_schema_name(self, value):
+        # https://wiki.apache.org/couchdb/HTTP_database_API
+        if value.lower() in self._ignore_dbs:
+            return False
+        if re.match(r'^[a-z]*$', value[0]):
+            return True
+        else:
+            return False
+
+    @classmethod
+    def _dict_requirements(cls):
+        return ['_name']
+
+
 class MySQLDatabase(Base):
     """Represents a Database and its properties."""
 
-    _ignore_dbs = CONF.ignore_dbs
-
     # Defaults
     __charset__ = "utf8"
     __collation__ = "utf8_general_ci"
@@ -400,6 +473,7 @@ class MySQLDatabase(Base):
         self._name = None
         self._collate = None
         self._character_set = None
+        self._ignore_dbs = cfg.get_ignored_dbs()
 
     @property
     def name(self):
@@ -459,6 +533,7 @@ class MySQLDatabase(Base):
 
 
 class ValidatedMySQLDatabase(MySQLDatabase):
+
     @MySQLDatabase.name.setter
     def name(self, value):
         if any([not value,
@@ -744,17 +819,115 @@ class MongoDBUser(DatastoreUser):
         return ['_name']
 
 
-class MySQLUser(Base):
-    """Represents a MySQL User and its associated properties."""
+class CassandraUser(DatastoreUser):
+    """Represents a Cassandra user and its associated properties."""
 
-    not_supported_chars = re.compile("^\s|\s$|'|\"|;|`|,|/|\\\\")
-    _ignore_users = CONF.ignore_users
+    def __init__(self, name=None, password=None, deserializing=False):
+        super(CassandraUser, self).__init__()
+
+        if ((not (bool(deserializing) != bool(name))) or
+                (bool(deserializing) and bool(password))):
+            raise ValueError(_("Bad args. name: %(name)s, "
+                               "password %(pass)s, "
+                               "deserializing %(deser)s.")
+                             % ({'name': bool(name),
+                                 'pass': bool(password),
+                                 'deser': bool(deserializing)}))
+        if not deserializing:
+            self.name = name
+            self.password = password
+
+    def _build_database_schema(self, name):
+        return CassandraSchema(name)
+
+    @property
+    def _max_username_length(self):
+        return 65535
+
+    def _is_valid_name(self, value):
+        return True
+
+    def _is_valid_host_name(self, value):
+        return True
+
+    def _is_valid_password(self, value):
+        return True
+
+    @classmethod
+    def _dict_requirements(cls):
+        return ['_name']
+
+
+class CouchDBUser(DatastoreUser):
+    """Represents a CouchDB user and its associated properties."""
 
     def __init__(self):
         self._name = None
         self._host = None
         self._password = None
         self._databases = []
+        self._ignore_users = cfg.get_ignored_users()
+
+    def _is_valid(self, value):
+        return True
+
+    @property
+    def name(self):
+        return self._name
+
+    @name.setter
+    def name(self, value):
+        if not self._is_valid(value):
+            raise ValueError(_("'%s' is not a valid user name.") % value)
+        else:
+            self._name = value
+
+    @property
+    def password(self):
+        return self._password
+
+    @password.setter
+    def password(self, value):
+        if not self._is_valid(value):
+            raise ValueError(_("'%s' is not a valid password.") % value)
+        else:
+            self._password = value
+
+    @property
+    def databases(self):
+        return self._databases
+
+    @databases.setter
+    def databases(self, value):
+        mydb = ValidatedMySQLDatabase()
+        mydb.name = value
+        self._databases.append(mydb.serialize())
+
+    @property
+    def host(self):
+        if self._host is None:
+            return '%'
+        return self._host
+
+    @host.setter
+    def host(self, value):
+        if not self._is_valid_host_name(value):
+            raise ValueError(_("'%s' is not a valid hostname.") % value)
+        else:
+            self._host = value
+
+
+class MySQLUser(Base):
+    """Represents a MySQL User and its associated properties."""
+
+    not_supported_chars = re.compile("^\s|\s$|'|\"|;|`|,|/|\\\\")
+
+    def __init__(self):
+        self._name = None
+        self._host = None
+        self._password = None
+        self._databases = []
+        self._ignore_users = cfg.get_ignored_users()
 
     def _is_valid(self, value):
         if (not value or
@@ -839,5 +1012,28 @@ class MySQLUser(Base):
 
 class RootUser(MySQLUser):
     """Overrides _ignore_users from the MySQLUser class."""
+    def __init__(self):
+        self._ignore_users = []
 
-    _ignore_users = []
+
+class MySQLRootUser(RootUser):
+    """Represents the MySQL root user."""
+
+    def __init__(self, password=None):
+        super(MySQLRootUser, self).__init__()
+        self._name = "root"
+        self._host = "%"
+        if password is None:
+            self._password = utils.generate_random_password()
+        else:
+            self._password = password
+
+
+class CassandraRootUser(CassandraUser):
+    """Represents the Cassandra default superuser."""
+
+    def __init__(self, password=None, *args, **kwargs):
+        if password is None:
+            password = utils.generate_random_password()
+        super(CassandraRootUser, self).__init__("cassandra", password=password,
+                                                *args, **kwargs)
