@@ -18,8 +18,8 @@ from eventlet.green import subprocess
 from oslo_log import log as logging
 
 from trove.common import cfg
+from trove.common.strategies.strategy import Strategy
 from trove.common import utils
-from trove.guestagent.strategy import Strategy
 
 LOG = logging.getLogger(__name__)
 CONF = cfg.CONF
