@@ -15,7 +15,6 @@
 #    under the License.
 
 """Model classes that form the core of instances functionality."""
-from __builtin__ import setattr
 from datetime import datetime
 from datetime import timedelta
 import re
@@ -29,6 +28,7 @@ from trove.common import cfg
 from trove.common import exception
 from trove.common import i18n as i18n
 import trove.common.instance as tr_instance
+from trove.common.notification import StartNotification
 from trove.common.remote import create_cinder_client
 from trove.common.remote import create_dns_client
 from trove.common.remote import create_guest_client
@@ -43,6 +43,8 @@ from trove.db import models as dbmodels
 from trove.extensions.security_group.models import SecurityGroup
 from trove.instance.tasks import InstanceTask
 from trove.instance.tasks import InstanceTasks
+from trove.module import models as module_models
+from trove.module import views as module_views
 from trove.quota.quota import run_with_quotas
 from trove.taskmanager import api as task_api
 
@@ -297,6 +299,8 @@ class SimpleInstance(object):
             return InstanceStatus.PROMOTE
         if InstanceTasks.EJECTING.action == action:
             return InstanceStatus.EJECT
+        if InstanceTasks.LOGGING.action == action:
+            return InstanceStatus.LOGGING
 
         # Check for server status.
         if self.db_info.server_status in ["BUILD", "ERROR", "REBOOT",
@@ -456,7 +460,7 @@ def load_any_instance(context, id, load_server=True):
         return load_instance(BuiltInstance, context, id,
                              needs_server=load_server)
     except exception.UnprocessableEntity:
-        LOG.warn(_LW("Could not load instance %s."), id)
+        LOG.warning(_LW("Could not load instance %s."), id)
         return load_instance(FreshInstance, context, id, needs_server=False)
 
 
@@ -569,7 +573,7 @@ class BaseInstance(SimpleInstance):
 
             if self.slaves:
                 msg = _("Detach replicas before deleting replica source.")
-                LOG.warn(msg)
+                LOG.warning(msg)
                 raise exception.ReplicaSourceDeleteForbidden(msg)
 
             self.update_db(task_status=InstanceTasks.DELETING,
@@ -668,8 +672,18 @@ class Instance(BuiltInstance):
     @classmethod
     def create(cls, context, name, flavor_id, image_id, databases, users,
                datastore, datastore_version, volume_size, backup_id,
-               availability_zone=None, nics=None, configuration_id=None,
-               slave_of_id=None, cluster_config=None, replica_count=None):
+               availability_zone=None, nics=None,
+               configuration_id=None, slave_of_id=None, cluster_config=None,
+               replica_count=None, volume_type=None, modules=None):
+
+        call_args = {
+            'name': name,
+            'flavor_id': flavor_id,
+            'datastore': datastore.name if datastore else None,
+            'datastore_version': datastore_version.name,
+            'image_id': image_id,
+            'availability_zone': availability_zone,
+        }
 
         # All nova flavors are permitted for a datastore-version unless one
         # or more entries are found in datastore_version_metadata,
@@ -682,7 +696,9 @@ class Instance(BuiltInstance):
             valid_flavors = tuple(f.value for f in bound_flavors)
             if flavor_id not in valid_flavors:
                 raise exception.DatastoreFlavorAssociationNotFound(
-                    version_id=datastore_version.id, flavor_id=flavor_id)
+                    datastore=datastore.name,
+                    datastore_version=datastore_version.name,
+                    flavor_id=flavor_id)
 
         datastore_cfg = CONF.get(datastore_version.manager)
         client = create_nova_client(context)
@@ -694,6 +710,7 @@ class Instance(BuiltInstance):
         deltas = {'instances': 1}
         volume_support = datastore_cfg.volume_support
         if volume_support:
+            call_args['volume_size'] = volume_size
             validate_volume_size(volume_size)
             deltas['volumes'] = volume_size
             # Instance volume should have enough space for the backup
@@ -709,6 +726,7 @@ class Instance(BuiltInstance):
                 target_size = flavor.ephemeral  # ephemeral_Storage
 
         if backup_id:
+            call_args['backup_id'] = backup_id
             backup_info = Backup.get_by_id(context, backup_id)
             if not backup_info.is_done_successfuly:
                 raise exception.BackupNotCompleteError(
@@ -731,6 +749,8 @@ class Instance(BuiltInstance):
                     datastore2=datastore.name)
 
         if slave_of_id:
+            call_args['replica_of'] = slave_of_id
+            call_args['replica_count'] = replica_count
             replication_support = datastore_cfg.replication_strategy
             if not replication_support:
                 raise exception.ReplicationNotSupported(
@@ -775,6 +795,27 @@ class Instance(BuiltInstance):
         if CONF.default_neutron_networks:
             nics = [{"net-id": net_id}
                     for net_id in CONF.default_neutron_networks] + nics
+        if nics:
+            call_args['nics'] = nics
+        if cluster_config:
+            call_args['cluster_id'] = cluster_config.get("id", None)
+
+        if not modules:
+            modules = []
+        module_ids = [mod['id'] for mod in modules]
+        modules = module_models.Modules.load_by_ids(context, module_ids)
+        auto_apply_modules = module_models.Modules.load_auto_apply(
+            context, datastore.id, datastore_version.id)
+        for aa_module in auto_apply_modules:
+            if aa_module.id not in module_ids:
+                modules.append(aa_module)
+        module_list = []
+        for module in modules:
+            module.contents = module_models.Module.deprocess_contents(
+                module.contents)
+            module_info = module_views.DetailedModuleView(module).data(
+                include_contents=True)
+            module_list.append(module_info)
 
         def _create_resources():
 
@@ -803,6 +844,7 @@ class Instance(BuiltInstance):
                           {'tenant': context.tenant, 'db': db_info.id})
 
                 instance_id = db_info.id
+                cls.add_instance_modules(context, instance_id, modules)
                 instance_name = name
                 ids.append(instance_id)
                 names.append(instance_name)
@@ -843,14 +885,20 @@ class Instance(BuiltInstance):
                 instance_id, instance_name, flavor, image_id, databases, users,
                 datastore_version.manager, datastore_version.packages,
                 volume_size, backup_id, availability_zone, root_password,
-                nics, overrides, slave_of_id, cluster_config)
+                nics, overrides, slave_of_id, cluster_config,
+                volume_type=volume_type, modules=module_list)
 
             return SimpleInstance(context, db_info, service_status,
                                   root_password)
 
-        return run_with_quotas(context.tenant,
-                               deltas,
-                               _create_resources)
+        with StartNotification(context, **call_args):
+            return run_with_quotas(context.tenant, deltas, _create_resources)
+
+    @classmethod
+    def add_instance_modules(cls, context, instance_id, modules):
+        for module in modules:
+            module_models.InstanceModule.create(
+                context, instance_id, module.id, module.md5)
 
     def get_flavor(self):
         client = create_nova_client(self.context)
@@ -1155,7 +1203,7 @@ class Instances(object):
     DEFAULT_LIMIT = CONF.instances_page_size
 
     @staticmethod
-    def load(context, include_clustered):
+    def load(context, include_clustered, instance_ids=None):
 
         def load_simple_instance(context, db, status, **kwargs):
             return SimpleInstance(context, db, status)
@@ -1164,17 +1212,19 @@ class Instances(object):
             raise TypeError("Argument context not defined.")
         client = create_nova_client(context)
         servers = client.servers.list()
-
-        if include_clustered:
-            db_infos = DBInstance.find_all(tenant_id=context.tenant,
-                                           deleted=False)
+        query_opts = {'tenant_id': context.tenant,
+                      'deleted': False}
+        if not include_clustered:
+            query_opts['cluster_id'] = None
+        if instance_ids and len(instance_ids) > 1:
+            raise exception.DatastoreOperationNotSupported(
+                operation='module-instances', datastore='current')
+            db_infos = DBInstance.query().filter_by(**query_opts)
         else:
-            db_infos = DBInstance.find_all(tenant_id=context.tenant,
-                                           cluster_id=None,
-                                           deleted=False)
-        limit = int(context.limit or Instances.DEFAULT_LIMIT)
-        if limit > Instances.DEFAULT_LIMIT:
-            limit = Instances.DEFAULT_LIMIT
+            if instance_ids:
+                query_opts['id'] = instance_ids[0]
+            db_infos = DBInstance.find_all(**query_opts)
+        limit = utils.pagination_limit(context.limit, Instances.DEFAULT_LIMIT)
         data_view = DBInstance.find_by_pagination('instances', db_infos, "foo",
                                                   limit=limit,
                                                   marker=context.marker)
