@@ -24,13 +24,18 @@ from trove.common import cfg
 from trove.common import exception
 from trove.common.i18n import _
 from trove.common.i18n import _LI
+from trove.common import notification
+from trove.common.notification import StartNotification
 from trove.common import pagination
+from trove.common.remote import create_guest_client
 from trove.common import utils
 from trove.common import wsgi
 from trove.datastore import models as datastore_models
 from trove.extensions.mysql.common import populate_users
 from trove.extensions.mysql.common import populate_validated_databases
 from trove.instance import models, views
+from trove.module import models as module_models
+from trove.module import views as module_views
 
 
 CONF = cfg.CONF
@@ -44,11 +49,11 @@ class InstanceController(wsgi.Controller):
 
     @classmethod
     def get_action_schema(cls, body, action_schema):
-        action_type = body.keys()[0]
+        action_type = list(body.keys())[0]
         action_schema = action_schema.get(action_type, {})
         if action_type == 'resize':
             # volume or flavorRef
-            resize_action = body[action_type].keys()[0]
+            resize_action = list(body[action_type].keys())[0]
             action_schema = action_schema.get(resize_action, {})
         return action_schema
 
@@ -92,13 +97,16 @@ class InstanceController(wsgi.Controller):
                      "instance %(instance_id)s for tenant '%(tenant_id)s'"),
                  {'action_name': action_name, 'instance_id': id,
                   'tenant_id': tenant_id})
-        return selected_action(instance, body)
+        return selected_action(context, req, instance, body)
 
-    def _action_restart(self, instance, body):
-        instance.restart()
+    def _action_restart(self, context, req, instance, body):
+        context.notification = notification.DBaaSInstanceRestart(context,
+                                                                 request=req)
+        with StartNotification(context, instance_id=instance.id):
+            instance.restart()
         return wsgi.Result(None, 202)
 
-    def _action_resize(self, instance, body):
+    def _action_resize(self, context, req, instance, body):
         """
         Handles 2 cases
         1. resize volume
@@ -119,26 +127,40 @@ class InstanceController(wsgi.Controller):
                 selected_option = options[key]
                 args = body['resize'][key]
                 break
-        return selected_option(instance, args)
+        return selected_option(context, req, instance, args)
 
-    def _action_resize_volume(self, instance, volume):
-        instance.resize_volume(volume['size'])
+    def _action_resize_volume(self, context, req, instance, volume):
+        context.notification = notification.DBaaSInstanceResizeVolume(
+            context, request=req)
+        with StartNotification(context, instance_id=instance.id,
+                               new_size=volume['size']):
+            instance.resize_volume(volume['size'])
         return wsgi.Result(None, 202)
 
-    def _action_resize_flavor(self, instance, flavorRef):
+    def _action_resize_flavor(self, context, req, instance, flavorRef):
+        context.notification = notification.DBaaSInstanceResizeInstance(
+            context, request=req)
         new_flavor_id = utils.get_id_from_href(flavorRef)
-        instance.resize_flavor(new_flavor_id)
+        with StartNotification(context, instance_id=instance.id,
+                               new_flavor_id=new_flavor_id):
+            instance.resize_flavor(new_flavor_id)
         return wsgi.Result(None, 202)
 
-    def _action_reset_password(self, instance, body):
+    def _action_reset_password(self, context, instance, body):
         raise webob.exc.HTTPNotImplemented()
 
-    def _action_promote_to_replica_source(self, instance, body):
-        instance.promote_to_replica_source()
+    def _action_promote_to_replica_source(self, context, req, instance, body):
+        context.notification = notification.DBaaSInstanceEject(context,
+                                                               request=req)
+        with StartNotification(context, instance_id=instance.id):
+            instance.promote_to_replica_source()
         return wsgi.Result(None, 202)
 
-    def _action_eject_replica_source(self, instance, body):
-        instance.eject_replica_source()
+    def _action_eject_replica_source(self, context, req, instance, body):
+        context.notification = notification.DBaaSInstancePromote(context,
+                                                                 request=req)
+        with StartNotification(context, instance_id=instance.id):
+            instance.eject_replica_source()
         return wsgi.Result(None, 202)
 
     def index(self, req, tenant_id):
@@ -185,11 +207,22 @@ class InstanceController(wsgi.Controller):
                      "'%(tenant_id)s'"),
                  {'instance_id': id, 'tenant_id': tenant_id})
         LOG.debug("req : '%s'\n\n", req)
-        # TODO(hub-cap): turn this into middleware
         context = req.environ[wsgi.CONTEXT_KEY]
         instance = models.load_any_instance(context, id)
-        instance.delete()
-        # TODO(cp16net): need to set the return code correctly
+        context.notification = notification.DBaaSInstanceDelete(
+            context, request=req)
+        with StartNotification(context, instance_id=instance.id):
+            marker = 'foo'
+            while marker:
+                instance_modules, marker = module_models.InstanceModules.load(
+                    context, instance_id=id)
+                for instance_module in instance_modules:
+                    instance_module = module_models.InstanceModule.load(
+                        context, instance_module['instance_id'],
+                        instance_module['module_id'])
+                    module_models.InstanceModule.delete(
+                        context, instance_module)
+            instance.delete()
         return wsgi.Result(None, 202)
 
     def create(self, req, body, tenant_id):
@@ -199,6 +232,8 @@ class InstanceController(wsgi.Controller):
         LOG.debug("req : '%s'\n\n", strutils.mask_password(req))
         LOG.debug("body : '%s'\n\n", strutils.mask_password(body))
         context = req.environ[wsgi.CONTEXT_KEY]
+        context.notification = notification.DBaaSInstanceCreate(context,
+                                                                request=req)
         datastore_args = body['instance'].get('datastore', {})
         datastore, datastore_version = (
             datastore_models.get_datastore_version(**datastore_args))
@@ -219,9 +254,12 @@ class InstanceController(wsgi.Controller):
             raise exception.BadRequest(msg=ve)
 
         if 'volume' in body['instance']:
-            volume_size = int(body['instance']['volume']['size'])
+            volume_info = body['instance']['volume']
+            volume_size = int(volume_info['size'])
+            volume_type = volume_info.get('type')
         else:
             volume_size = None
+            volume_type = None
 
         if 'restorePoint' in body['instance']:
             backupRef = body['instance']['restorePoint']['backupRef']
@@ -236,13 +274,16 @@ class InstanceController(wsgi.Controller):
                                            # also check for older name
                                            body['instance'].get('slave_of'))
         replica_count = body['instance'].get('replica_count')
+        modules = body['instance'].get('modules')
         instance = models.Instance.create(context, name, flavor_id,
                                           image_id, databases, users,
                                           datastore, datastore_version,
                                           volume_size, backup_id,
                                           availability_zone, nics,
                                           configuration, slave_of_id,
-                                          replica_count=replica_count)
+                                          replica_count=replica_count,
+                                          volume_type=volume_type,
+                                          modules=modules)
 
         view = views.InstanceDetailView(instance, req=req)
         return wsgi.Result(view.data(), 200)
@@ -254,7 +295,7 @@ class InstanceController(wsgi.Controller):
                 configuration_id = utils.get_id_from_href(configuration_ref)
                 return configuration_id
 
-    def _modify_instance(self, instance, **kwargs):
+    def _modify_instance(self, context, req, instance, **kwargs):
         """Modifies the instance using the specified keyword arguments
         'detach_replica': ignored if not present or False, if True,
         specifies the instance is a replica that will be detached from
@@ -266,12 +307,25 @@ class InstanceController(wsgi.Controller):
 
         if 'detach_replica' in kwargs and kwargs['detach_replica']:
             LOG.debug("Detaching replica from source.")
-            instance.detach_replica()
+            context.notification = notification.DBaaSInstanceDetach(
+                context, request=req)
+            with StartNotification(context, instance_id=instance.id):
+                instance.detach_replica()
         if 'configuration_id' in kwargs:
             if kwargs['configuration_id']:
-                instance.assign_configuration(kwargs['configuration_id'])
+                context.notification = (
+                    notification.DBaaSInstanceAttachConfiguration(context,
+                                                                  request=req))
+                configuration_id = kwargs['configuration_id']
+                with StartNotification(context, instance_id=instance.id,
+                                       configuration_id=configuration_id):
+                    instance.assign_configuration(configuration_id)
             else:
-                instance.unassign_configuration()
+                context.notification = (
+                    notification.DBaaSInstanceDetachConfiguration(context,
+                                                                  request=req))
+                with StartNotification(context, instance_id=instance.id):
+                    instance.unassign_configuration()
         if kwargs:
             instance.update_db(**kwargs)
 
@@ -289,7 +343,7 @@ class InstanceController(wsgi.Controller):
         # Make sure args contains a 'configuration_id' argument,
         args = {}
         args['configuration_id'] = self._configuration_parse(context, body)
-        self._modify_instance(instance, **args)
+        self._modify_instance(context, req, instance, **args)
         return wsgi.Result(None, 202)
 
     def edit(self, req, id, body, tenant_id):
@@ -311,7 +365,7 @@ class InstanceController(wsgi.Controller):
             args['name'] = body['instance']['name']
         if 'configuration' in body['instance']:
             args['configuration_id'] = self._configuration_parse(context, body)
-        self._modify_instance(instance, **args)
+        self._modify_instance(context, req, instance, **args)
         return wsgi.Result(None, 202)
 
     def configuration(self, req, tenant_id, id):
@@ -327,3 +381,96 @@ class InstanceController(wsgi.Controller):
                   {'instance_id': id, 'config': config})
         return wsgi.Result(views.DefaultConfigurationView(
                            config).data(), 200)
+
+    def guest_log_list(self, req, tenant_id, id):
+        """Return all information about all logs for an instance."""
+        LOG.debug("Listing logs for tenant %s" % tenant_id)
+        context = req.environ[wsgi.CONTEXT_KEY]
+        instance = models.Instance.load(context, id)
+        if not instance:
+            raise exception.NotFound(uuid=id)
+        client = create_guest_client(context, id)
+        guest_log_list = client.guest_log_list()
+        return wsgi.Result({'logs': guest_log_list}, 200)
+
+    def guest_log_action(self, req, body, tenant_id, id):
+        """Processes a guest log."""
+        LOG.info(_("Processing log for tenant %s"), tenant_id)
+        context = req.environ[wsgi.CONTEXT_KEY]
+        instance = models.Instance.load(context, id)
+        if not instance:
+            raise exception.NotFound(uuid=id)
+        log_name = body['name']
+        enable = body.get('enable', None)
+        disable = body.get('disable', None)
+        publish = body.get('publish', None)
+        discard = body.get('discard', None)
+        if enable and disable:
+            raise exception.BadRequest(_("Cannot enable and disable log."))
+        client = create_guest_client(context, id)
+        guest_log = client.guest_log_action(log_name, enable, disable,
+                                            publish, discard)
+        return wsgi.Result({'log': guest_log}, 200)
+
+    def module_list(self, req, tenant_id, id):
+        """Return information about modules on an instance."""
+        context = req.environ[wsgi.CONTEXT_KEY]
+        instance = models.Instance.load(context, id)
+        if not instance:
+            raise exception.NotFound(uuid=id)
+        from_guest = bool(req.GET.get('from_guest', '').lower())
+        include_contents = bool(req.GET.get('include_contents', '').lower())
+        if from_guest:
+            return self._module_list_guest(
+                context, id, include_contents=include_contents)
+        else:
+            return self._module_list(
+                context, id, include_contents=include_contents)
+
+    def _module_list_guest(self, context, id, include_contents):
+        """Return information about modules on an instance."""
+        client = create_guest_client(context, id)
+        result_list = client.module_list(include_contents)
+        return wsgi.Result({'modules': result_list}, 200)
+
+    def _module_list(self, context, id, include_contents):
+        """Return information about instnace modules."""
+        client = create_guest_client(context, id)
+        result_list = client.module_list(include_contents)
+        return wsgi.Result({'modules': result_list}, 200)
+
+    def module_apply(self, req, body, tenant_id, id):
+        """Apply modules to an instance."""
+        context = req.environ[wsgi.CONTEXT_KEY]
+        instance = models.Instance.load(context, id)
+        if not instance:
+            raise exception.NotFound(uuid=id)
+        module_ids = [mod['id'] for mod in body.get('modules', [])]
+        modules = module_models.Modules.load_by_ids(context, module_ids)
+        module_list = []
+        for module in modules:
+            module.contents = module_models.Module.deprocess_contents(
+                module.contents)
+            module_info = module_views.DetailedModuleView(module).data(
+                include_contents=True)
+            module_list.append(module_info)
+        client = create_guest_client(context, id)
+        result_list = client.module_apply(module_list)
+        models.Instance.add_instance_modules(context, id, modules)
+        return wsgi.Result({'modules': result_list}, 200)
+
+    def module_remove(self, req, tenant_id, id, module_id):
+        """Remove module from an instance."""
+        context = req.environ[wsgi.CONTEXT_KEY]
+        instance = models.Instance.load(context, id)
+        if not instance:
+            raise exception.NotFound(uuid=id)
+        module = module_models.Module.load(context, module_id)
+        module_info = module_views.DetailedModuleView(module).data()
+        client = create_guest_client(context, id)
+        client.module_remove(module_info)
+        instance_module = module_models.InstanceModule.load(
+            context, instance_id=id, module_id=module_id)
+        if instance_module:
+            module_models.InstanceModule.delete(context, instance_module)
+        return wsgi.Result(None, 200)
