@@ -43,6 +43,11 @@ from trove.common.exception import VolumeCreationFailure
 from trove.common.i18n import _
 from trove.common import instance as rd_instance
 from trove.common.instance import ServiceStatuses
+from trove.common.notification import (
+    TroveInstanceCreate,
+    TroveInstanceModifyVolume,
+    TroveInstanceModifyFlavor,
+    TroveInstanceDelete)
 import trove.common.remote as remote
 from trove.common.remote import create_cinder_client
 from trove.common.remote import create_dns_client
@@ -92,7 +97,7 @@ class NotifyMixin(object):
             datastore_manager_id = id_map[datastore_manager]
         else:
             datastore_manager_id = cfg.UNKNOWN_SERVICE_ID
-            LOG.error("Datastore ID for Manager (%s) is not configured"
+            LOG.error(_("Datastore ID for Manager (%s) is not configured")
                       % datastore_manager)
         return datastore_manager_id
 
@@ -181,7 +186,8 @@ class ConfigurationMixin(object):
 
 class ClusterTasks(Cluster):
 
-    def update_statuses_on_failure(self, cluster_id, shard_id=None):
+    def update_statuses_on_failure(self, cluster_id, shard_id=None,
+                                   status=None):
 
         if CONF.update_status_on_fail:
             if shard_id:
@@ -193,7 +199,7 @@ class ClusterTasks(Cluster):
 
             for db_instance in db_instances:
                 db_instance.set_task_status(
-                    InstanceTasks.BUILDING_ERROR_SERVER)
+                    status or InstanceTasks.BUILDING_ERROR_SERVER)
                 db_instance.save()
 
     @classmethod
@@ -202,59 +208,85 @@ class ClusterTasks(Cluster):
 
     def _all_instances_ready(self, instance_ids, cluster_id,
                              shard_id=None):
+        """Wait for all instances to get READY."""
+        return self._all_instances_acquire_status(
+            instance_ids, cluster_id, shard_id, ServiceStatuses.INSTANCE_READY,
+            fast_fail_statuses=[ServiceStatuses.FAILED,
+                                ServiceStatuses.FAILED_TIMEOUT_GUESTAGENT])
 
-        def _all_status_ready(ids):
-            LOG.debug("Checking service status of instance ids: %s" % ids)
+    def _all_instances_shutdown(self, instance_ids, cluster_id,
+                                shard_id=None):
+        """Wait for all instances to go SHUTDOWN."""
+        return self._all_instances_acquire_status(
+            instance_ids, cluster_id, shard_id, ServiceStatuses.SHUTDOWN,
+            fast_fail_statuses=[ServiceStatuses.FAILED,
+                                ServiceStatuses.FAILED_TIMEOUT_GUESTAGENT])
+
+    def _all_instances_running(self, instance_ids, cluster_id, shard_id=None):
+        """Wait for all instances to become ACTIVE."""
+        return self._all_instances_acquire_status(
+            instance_ids, cluster_id, shard_id, ServiceStatuses.RUNNING,
+            fast_fail_statuses=[ServiceStatuses.FAILED,
+                                ServiceStatuses.FAILED_TIMEOUT_GUESTAGENT])
+
+    def _all_instances_acquire_status(
+            self, instance_ids, cluster_id, shard_id, expected_status,
+            fast_fail_statuses=None):
+
+        def _is_fast_fail_status(status):
+            return ((fast_fail_statuses is not None) and
+                    ((status == fast_fail_statuses) or
+                     (status in fast_fail_statuses)))
+
+        def _all_have_status(ids):
             for instance_id in ids:
                 status = InstanceServiceStatus.find_by(
                     instance_id=instance_id).get_status()
-                if (status == ServiceStatuses.FAILED or
-                   status == ServiceStatuses.FAILED_TIMEOUT_GUESTAGENT):
-                        # if one has failed, no need to continue polling
-                        LOG.debug("Instance %s in %s, exiting polling." % (
-                            instance_id, status))
-                        return True
-                if status != ServiceStatuses.BUILD_PENDING:
-                        # if one is not in a cluster-ready state,
-                        # continue polling
-                        LOG.debug("Instance %s in %s, continue polling." % (
-                            instance_id, status))
-                        return False
-            LOG.debug("Instances are ready, exiting polling for: %s" % ids)
+                if _is_fast_fail_status(status):
+                    # if one has failed, no need to continue polling
+                    LOG.debug("Instance %s has acquired a fast-fail status %s."
+                              % (instance_id, status))
+                    return True
+                if status != expected_status:
+                    # if one is not in the expected state, continue polling
+                    LOG.debug("Instance %s was %s." % (instance_id, status))
+                    return False
+
             return True
 
         def _instance_ids_with_failures(ids):
-            LOG.debug("Checking for service status failures for "
-                      "instance ids: %s" % ids)
+            LOG.debug("Checking for service failures on instances: %s"
+                      % ids)
             failed_instance_ids = []
             for instance_id in ids:
                 status = InstanceServiceStatus.find_by(
                     instance_id=instance_id).get_status()
-                if (status == ServiceStatuses.FAILED or
-                   status == ServiceStatuses.FAILED_TIMEOUT_GUESTAGENT):
-                        failed_instance_ids.append(instance_id)
+                if _is_fast_fail_status(status):
+                    failed_instance_ids.append(instance_id)
             return failed_instance_ids
 
-        LOG.debug("Polling until service status is ready for "
-                  "instance ids: %s" % instance_ids)
+        LOG.debug("Polling until all instances acquire %s status: %s"
+                  % (expected_status, instance_ids))
         try:
             utils.poll_until(lambda: instance_ids,
-                             lambda ids: _all_status_ready(ids),
+                             lambda ids: _all_have_status(ids),
                              sleep_time=USAGE_SLEEP_TIME,
                              time_out=CONF.usage_timeout)
         except PollTimeOut:
-            LOG.exception(_("Timeout for all instance service statuses "
-                            "to become ready."))
+            LOG.exception(_("Timed out while waiting for all instances "
+                            "to become %s.") % expected_status)
             self.update_statuses_on_failure(cluster_id, shard_id)
             return False
 
         failed_ids = _instance_ids_with_failures(instance_ids)
         if failed_ids:
-            LOG.error(_("Some instances failed to become ready: %s") %
-                      failed_ids)
+            LOG.error(_("Some instances failed: %s") % failed_ids)
             self.update_statuses_on_failure(cluster_id, shard_id)
             return False
 
+        LOG.debug("All instances have acquired the expected status %s."
+                  % expected_status)
+
         return True
 
     def delete_cluster(self, context, cluster_id):
@@ -320,7 +352,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                              sleep_time=USAGE_SLEEP_TIME,
                              time_out=timeout)
             LOG.info(_("Created instance %s successfully.") % self.id)
-            self.send_usage_event('create', instance_size=flavor['ram'])
+            TroveInstanceCreate(instance=self,
+                                instance_size=flavor['ram']).notify()
         except PollTimeOut:
             LOG.error(_("Failed to create instance %s. "
                         "Timeout waiting for instance to become active. "
@@ -333,7 +366,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
     def create_instance(self, flavor, image_id, databases, users,
                         datastore_manager, packages, volume_size,
                         backup_id, availability_zone, root_password, nics,
-                        overrides, cluster_config, snapshot=None):
+                        overrides, cluster_config, snapshot, volume_type,
+                        modules):
         # It is the caller's responsibility to ensure that
         # FreshInstanceTasks.wait_for_instance is called after
         # create_instance to ensure that the proper usage event gets sent
@@ -359,7 +393,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                           "instance: %s" % self.id)
 
         files = self._get_injected_files(datastore_manager)
-
+        cinder_volume_type = volume_type or CONF.cinder_volume_type
         if use_heat:
             volume_info = self._create_server_volume_heat(
                 flavor,
@@ -368,7 +402,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                 volume_size,
                 availability_zone,
                 nics,
-                files)
+                files,
+                cinder_volume_type)
         elif use_nova_server_volume:
             volume_info = self._create_server_volume(
                 flavor['id'],
@@ -388,7 +423,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                 volume_size,
                 availability_zone,
                 nics,
-                files)
+                files,
+                cinder_volume_type)
 
         config = self._render_config(flavor)
 
@@ -396,6 +432,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
         if backup_id is not None:
                 backup = bkup_models.Backup.get_by_id(self.context, backup_id)
                 backup_info = {'id': backup_id,
+                               'instance_id': backup.instance_id,
                                'location': backup.location,
                                'type': backup.backup_type,
                                'checksum': backup.checksum,
@@ -404,7 +441,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                             packages, databases, users, backup_info,
                             config.config_contents, root_password,
                             overrides,
-                            cluster_config, snapshot)
+                            cluster_config, snapshot, modules)
 
         if root_password:
             self.report_root_enabled()
@@ -542,22 +579,22 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
             service.set_status(ServiceStatuses.
                                FAILED_TIMEOUT_GUESTAGENT)
             service.save()
-            LOG.error(_("Service status: %(status)s") %
+            LOG.error(_("Service status: %(status)s\n"
+                        "Service error description: %(desc)s") %
                       {'status': ServiceStatuses.
-                       FAILED_TIMEOUT_GUESTAGENT.api_status})
-            LOG.error(_("Service error description: %(desc)s") %
-                      {'desc': ServiceStatuses.
+                       FAILED_TIMEOUT_GUESTAGENT.api_status,
+                       'desc': ServiceStatuses.
                        FAILED_TIMEOUT_GUESTAGENT.description})
             # Updating instance status
             db_info = DBInstance.find_by(id=self.id, deleted=False)
             db_info.set_task_status(InstanceTasks.
                                     BUILDING_ERROR_TIMEOUT_GA)
             db_info.save()
-            LOG.error(_("Trove instance status: %(action)s") %
+            LOG.error(_("Trove instance status: %(action)s\n"
+                        "Trove instance status description: %(text)s") %
                       {'action': InstanceTasks.
-                       BUILDING_ERROR_TIMEOUT_GA.action})
-            LOG.error(_("Trove instance status description: %(text)s") %
-                      {'text': InstanceTasks.
+                       BUILDING_ERROR_TIMEOUT_GA.action,
+                       'text': InstanceTasks.
                        BUILDING_ERROR_TIMEOUT_GA.db_text})
 
     def _service_is_active(self):
@@ -574,7 +611,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
         service = InstanceServiceStatus.find_by(instance_id=self.id)
         status = service.get_status()
         if (status == rd_instance.ServiceStatuses.RUNNING or
-           status == rd_instance.ServiceStatuses.BUILD_PENDING):
+           status == rd_instance.ServiceStatuses.INSTANCE_READY):
                 return True
         elif status not in [rd_instance.ServiceStatuses.NEW,
                             rd_instance.ServiceStatuses.BUILDING]:
@@ -606,12 +643,12 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                 availability_zone=availability_zone,
                 nics=nics, config_drive=config_drive,
                 userdata=userdata)
-            LOG.debug("Created new compute instance %(server_id)s "
-                      "for id: %(id)s" %
-                      {'server_id': server.id, 'id': self.id})
-
             server_dict = server._info
-            LOG.debug("Server response: %s" % server_dict)
+            LOG.debug("Created new compute instance %(server_id)s "
+                      "for id: %(id)s\nServer response: %(response)s" %
+                      {'server_id': server.id, 'id': self.id,
+                       'response': server_dict})
+
             volume_id = None
             for volume in server_dict.get('os:volumes', []):
                 volume_id = volume.get('id')
@@ -643,7 +680,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
 
     def _create_server_volume_heat(self, flavor, image_id,
                                    datastore_manager, volume_size,
-                                   availability_zone, nics, files):
+                                   availability_zone, nics, files,
+                                   volume_type):
         LOG.debug("Begin _create_server_volume_heat for id: %s" % self.id)
         try:
             client = create_heat_client(self.context)
@@ -668,6 +706,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
 
             parameters = {"Flavor": flavor["name"],
                           "VolumeSize": volume_size,
+                          "VolumeType": volume_type,
                           "InstanceId": self.id,
                           "ImageId": image_id,
                           "DatastoreManager": datastore_manager,
@@ -739,12 +778,13 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
     def _create_server_volume_individually(self, flavor_id, image_id,
                                            security_groups, datastore_manager,
                                            volume_size, availability_zone,
-                                           nics, files):
+                                           nics, files, volume_type):
         LOG.debug("Begin _create_server_volume_individually for id: %s" %
                   self.id)
         server = None
         volume_info = self._build_volume_info(datastore_manager,
-                                              volume_size=volume_size)
+                                              volume_size=volume_size,
+                                              volume_type=volume_type)
         block_device_mapping = volume_info['block_device']
         try:
             server = self._create_server(flavor_id, image_id, security_groups,
@@ -762,7 +802,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                   self.id)
         return volume_info
 
-    def _build_volume_info(self, datastore_manager, volume_size=None):
+    def _build_volume_info(self, datastore_manager, volume_size=None,
+                           volume_type=None):
         volume_info = None
         volume_support = self.volume_support
         device_path = self.device_path
@@ -771,14 +812,18 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
         if volume_support:
             try:
                 volume_info = self._create_volume(
-                    volume_size, datastore_manager)
+                    volume_size, volume_type, datastore_manager)
             except Exception as e:
                 msg = _("Failed to create volume for instance %s") % self.id
                 err = inst_models.InstanceTasks.BUILDING_ERROR_VOLUME
                 self._log_and_raise(e, msg, err)
         else:
-            LOG.debug("device_path = %s" % device_path)
-            LOG.debug("mount_point = %s" % mount_point)
+            LOG.debug("device_path = %(path)s\n"
+                      "mount_point = %(point)s" %
+                      {
+                          "path": device_path,
+                          "point": mount_point
+                      })
             volume_info = {
                 'block_device': None,
                 'device_path': device_path,
@@ -788,19 +833,21 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
         return volume_info
 
     def _log_and_raise(self, exc, message, task_status):
-        LOG.error(message)
-        LOG.error(exc)
-        LOG.error(traceback.format_exc())
+        LOG.error(_("%(message)s\n%(exc)s\n%(trace)s") %
+                  {"message": message,
+                   "exc": exc,
+                   "trace": traceback.format_exc()})
         self.update_db(task_status=task_status)
         raise TroveError(message=message)
 
-    def _create_volume(self, volume_size, datastore_manager):
+    def _create_volume(self, volume_size, volume_type, datastore_manager):
         LOG.debug("Begin _create_volume for id: %s" % self.id)
         volume_client = create_cinder_client(self.context)
         volume_desc = ("datastore volume for %s" % self.id)
         volume_ref = volume_client.volumes.create(
             volume_size, name="datastore-%s" % self.id,
-            description=volume_desc, volume_type=CONF.cinder_volume_type)
+            description=volume_desc,
+            volume_type=volume_type)
 
         # Record the volume ID in case something goes wrong.
         self.update_db(volume_id=volume_ref.id)
@@ -827,13 +874,18 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
         block_device = {bdm: mapping}
         created_volumes = [{'id': v_ref.id,
                             'size': v_ref.size}]
-        LOG.debug("block_device = %s" % block_device)
-        LOG.debug("volume = %s" % created_volumes)
 
         device_path = self.device_path
         mount_point = CONF.get(datastore_manager).mount_point
-        LOG.debug("device_path = %s" % device_path)
-        LOG.debug("mount_point = %s" % mount_point)
+
+        LOG.debug("block_device = %(device)s\n"
+                  "volume = %(volume)s\n"
+                  "device_path = %(path)s\n"
+                  "mount_point = %(point)s" %
+                  {"device": block_device,
+                   "volume": created_volumes,
+                   "path": device_path,
+                   "point": mount_point})
 
         volume_info = {'block_device': block_device,
                        'device_path': device_path,
@@ -871,7 +923,8 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
     def _guest_prepare(self, flavor_ram, volume_info,
                        packages, databases, users, backup_info=None,
                        config_contents=None, root_password=None,
-                       overrides=None, cluster_config=None, snapshot=None):
+                       overrides=None, cluster_config=None, snapshot=None,
+                       modules=None):
         LOG.debug("Entering guest_prepare")
         # Now wait for the response from the create to do additional work
         self.guest.prepare(flavor_ram, packages, databases, users,
@@ -882,7 +935,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
                            root_password=root_password,
                            overrides=overrides,
                            cluster_config=cluster_config,
-                           snapshot=snapshot)
+                           snapshot=snapshot, modules=modules)
 
     def _create_dns_entry(self):
         dns_support = CONF.trove_dns_support
@@ -917,7 +970,7 @@ class FreshInstanceTasks(FreshInstance, NotifyMixin, ConfigurationMixin):
             server = self.nova_client.servers.get(
                 self.db_info.compute_instance_id)
             self.db_info.addresses = server.addresses
-            LOG.debug(_("Creating dns entry..."))
+            LOG.debug("Creating dns entry...")
             ip = self.dns_ip_address
             if not ip:
                 raise TroveError("Failed to create DNS entry for instance %s. "
@@ -1055,9 +1108,9 @@ class BuiltInstanceTasks(BuiltInstance, NotifyMixin, ConfigurationMixin):
             LOG.exception(_("Error deleting volume of instance %(id)s.") %
                           {'id': self.db_info.id})
 
-        self.send_usage_event('delete',
-                              deleted_at=timeutils.isotime(deleted_at),
-                              server=old_server)
+        TroveInstanceDelete(instance=self,
+                            deleted_at=timeutils.isotime(deleted_at),
+                            server=old_server).notify()
         LOG.debug("End _delete_resources for instance %s" % self.id)
 
     def server_status_matches(self, expected_status, server=None):
@@ -1256,6 +1309,27 @@ class BuiltInstanceTasks(BuiltInstance, NotifyMixin, ConfigurationMixin):
         finally:
             self.reset_task_status()
 
+    def guest_log_list(self):
+        LOG.info(_("Retrieving guest log list for instance %s.") % self.id)
+        try:
+            return self.guest.guest_log_list()
+        except GuestError:
+            LOG.error(_("Failed to retrieve guest log list for instance "
+                        "%s.") % self.id)
+        finally:
+            self.reset_task_status()
+
+    def guest_log_action(self, log_name, enable, disable, publish, discard):
+        LOG.info(_("Processing guest log for instance %s.") % self.id)
+        try:
+            return self.guest.guest_log_action(log_name, enable, disable,
+                                               publish, discard)
+        except GuestError:
+            LOG.error(_("Failed to process guest log for instance %s.")
+                      % self.id)
+        finally:
+            self.reset_task_status()
+
     def refresh_compute_server_info(self):
         """Refreshes the compute server field."""
         server = self.nova_client.servers.get(self.server.id)
@@ -1564,11 +1638,12 @@ class ResizeVolumeAction(object):
                 self.instance.volume_id)
             launched_time = timeutils.isotime(self.instance.updated)
             modified_time = timeutils.isotime(self.instance.updated)
-            self.instance.send_usage_event('modify_volume',
-                                           old_volume_size=self.old_size,
-                                           launched_at=launched_time,
-                                           modify_at=modified_time,
-                                           volume_size=volume.size)
+            TroveInstanceModifyVolume(instance=self.instance,
+                                      old_volume_size=self.old_size,
+                                      launched_at=launched_time,
+                                      modify_at=modified_time,
+                                      volume_size=volume.size,
+                                      ).notify()
         else:
             self.instance.reset_task_status()
             msg = _("Failed to resize instance %(id)s volume for server "
@@ -1762,9 +1837,9 @@ class ResizeAction(ResizeActionBase):
         self.instance.server.resize(self.new_flavor_id)
 
     def _revert_nova_action(self):
-        LOG.debug("Instance %s calling Compute revert resize..."
+        LOG.debug("Instance %s calling Compute revert resize... "
+                  "Repairing config."
                   % self.instance.id)
-        LOG.debug("Repairing config.")
         try:
             config = self.instance._render_config(self.old_flavor)
             config = {'config_contents': config.config_contents}
@@ -1779,13 +1854,13 @@ class ResizeAction(ResizeActionBase):
                   % {'id': self.instance.id, 'flavor_id': self.new_flavor_id})
         self.instance.update_db(flavor_id=self.new_flavor_id,
                                 task_status=inst_models.InstanceTasks.NONE)
-        self.instance.send_usage_event(
-            'modify_flavor',
-            old_instance_size=self.old_flavor['ram'],
-            instance_size=self.new_flavor['ram'],
-            launched_at=timeutils.isotime(self.instance.updated),
-            modify_at=timeutils.isotime(self.instance.updated),
-            server=self.instance.server)
+        update_time = timeutils.isotime(self.instance.updated)
+        TroveInstanceModifyFlavor(instance=self.instance,
+                                  old_instance_size=self.old_flavor['ram'],
+                                  instance_size=self.new_flavor['ram'],
+                                  launched_at=update_time,
+                                  modify_at=update_time,
+                                  server=self.instance.server).notify()
 
     def _start_datastore(self):
         config = self.instance._render_config(self.new_flavor)
@@ -1802,9 +1877,11 @@ class MigrateAction(ResizeActionBase):
         LOG.debug("Currently no assertions for a Migrate Action")
 
     def _initiate_nova_action(self):
-        LOG.debug("Migrating instance %s without flavor change ..."
-                  % self.instance.id)
-        LOG.debug("Forcing migration to host(%s)" % self.host)
+        LOG.debug("Migrating instance %(instance)s without flavor change ...\n"
+                  "Forcing migration to host(%(host)s)" %
+                  {"instance": self.instance.id,
+                   "host": self.host})
+
         self.instance.server.migrate(force_host=self.host)
 
     def _record_action_success(self):
