@@ -67,8 +67,7 @@ GROUP_SECURITY_GROUPS = "dbaas.api.security_groups"
 GROUP_CREATE_INSTANCE_FAILURE = "dbaas.api.failures"
 GROUP_QUOTAS = "dbaas.quotas"
 
-# FIXME(amrith) This is temporary fix - set timeout to 62 minutes
-TIMEOUT_INSTANCE_CREATE = 60 * 62
+TIMEOUT_INSTANCE_CREATE = 60 * 32
 TIMEOUT_INSTANCE_DELETE = 120
 
 
@@ -86,6 +85,7 @@ class InstanceTestInfo(object):
         self.id = None  # The ID of the instance in the database.
         self.local_id = None
         self.address = None
+        self.nics = None  # The dict of type/id for nics used on the intance.
         self.initial_result = None  # The initial result from the create call.
         self.user_ip = None  # The IP address of the instance, given to user.
         self.infra_ip = None  # The infrastructure network IP address.
@@ -240,8 +240,8 @@ class CreateInstanceQuotaTest(unittest.TestCase):
         self.test_info.dbaas_datastore = CONFIG.dbaas_datastore
 
     def tearDown(self):
-        quota_dict = {'instances': CONFIG.trove_max_instances_per_user,
-                      'volumes': CONFIG.trove_max_volumes_per_user}
+        quota_dict = {'instances': CONFIG.trove_max_instances_per_tenant,
+                      'volumes': CONFIG.trove_max_volumes_per_tenant}
         dbaas_admin.quota.update(self.test_info.user.tenant_id,
                                  quota_dict)
 
@@ -283,7 +283,7 @@ class CreateInstanceQuotaTest(unittest.TestCase):
         self.test_info.volume = None
 
         if VOLUME_SUPPORT:
-            assert_equal(CONFIG.trove_max_volumes_per_user,
+            assert_equal(CONFIG.trove_max_volumes_per_tenant,
                          verify_quota['volumes'])
             self.test_info.volume = {'size':
                                      CONFIG.get('trove_volume_size', 1)}
@@ -657,6 +657,10 @@ class CreateInstance(object):
         else:
             instance_info.volume = None
 
+        shared_network = CONFIG.get('shared_network', None)
+        if shared_network:
+            instance_info.nics = [{'net-id': shared_network}]
+
         if create_new_instance():
             instance_info.initial_result = dbaas.instances.create(
                 instance_info.name,
@@ -664,6 +668,7 @@ class CreateInstance(object):
                 instance_info.volume,
                 databases,
                 users,
+                nics=instance_info.nics,
                 availability_zone="nova",
                 datastore=instance_info.dbaas_datastore,
                 datastore_version=instance_info.dbaas_datastore_version)
