@@ -43,6 +43,7 @@ class SlaveInstanceTestInfo(object):
 GROUP = "dbaas.api.replication"
 slave_instance = SlaveInstanceTestInfo()
 existing_db_on_master = generate_uuid()
+backup_count = None
 
 
 def _get_user_count(server_info):
@@ -66,6 +67,15 @@ def slave_is_running(running=True):
     return check_slave_is_running
 
 
+def backup_count_matches(count):
+
+    def check_backup_count_matches():
+        backup = instance_info.dbaas.instances.backups(instance_info.id)
+        return count == len(backup)
+
+    return check_backup_count_matches
+
+
 def instance_is_active(id):
     instance = instance_info.dbaas.instances.get(id)
     if instance.status == "ACTIVE":
@@ -80,6 +90,9 @@ def create_slave():
         instance_info.name + "_slave",
         instance_info.dbaas_flavor_href,
         instance_info.volume,
+        nics=instance_info.nics,
+        datastore=instance_info.dbaas_datastore,
+        datastore_version=instance_info.dbaas_datastore_version,
         slave_of=instance_info.id)
     assert_equal(200, instance_info.dbaas.last_http_code)
     assert_equal("BUILD", result.status)
@@ -115,6 +128,8 @@ class CreateReplicationSlave(object):
                       instance_info.name + "_slave",
                       instance_info.dbaas_flavor_href,
                       instance_info.volume,
+                      datastore=instance_info.dbaas_datastore,
+                      datastore_version=instance_info.dbaas_datastore_version,
                       slave_of="Missing replica source")
         assert_equal(404, instance_info.dbaas.last_http_code)
 
@@ -128,6 +143,9 @@ class CreateReplicationSlave(object):
 
     @test(runs_after=['test_create_db_on_master'])
     def test_create_slave(self):
+        global backup_count
+        backup_count = len(
+            instance_info.dbaas.instances.backups(instance_info.id))
         slave_instance.id = create_slave()
 
 
@@ -161,9 +179,9 @@ class VerifySlave(object):
         poll_until(slave_is_running())
 
     @test(runs_after=[test_correctly_started_replication])
+    @time_out(60)
     def test_backup_deleted(self):
-        backup = instance_info.dbaas.instances.backups(instance_info.id)
-        assert_equal(len(backup), 0)
+        poll_until(backup_count_matches(backup_count))
 
     @test(depends_on=[test_correctly_started_replication])
     def test_slave_is_read_only(self):
