@@ -13,7 +13,8 @@
 #    under the License.
 
 import time
-import urllib
+
+from six.moves.urllib import parse as urllib_parse
 
 from proboscis import after_class
 from proboscis.asserts import assert_equal
@@ -24,7 +25,6 @@ from proboscis.asserts import fail
 from proboscis import before_class
 from proboscis import test
 from troveclient.compat import exceptions
-from troveclient.openstack.common.apiclient.exceptions import ValidationError
 
 from trove import tests
 from trove.tests.api.databases import TestDatabases
@@ -309,23 +309,6 @@ class TestUsers(object):
         assert_equal(400, self.dbaas.last_http_code)
         self.dbaas.users.delete(instance_info.id, username, hostname=hostname)
 
-    @test()
-    def test_updateuser_nochanges(self):
-        # Cannot update the user without passing in at least one change
-        users = []
-        username = "testuser_nochg"
-        hostname = "192.168.0.1"
-        users.append({"name": username, "password": "password",
-                      "host": hostname, "databases": []})
-        self.dbaas.users.create(instance_info.id, users)
-        user_new = {}
-        assert_raises(ValidationError,
-                      self.dbaas.users.update_attributes, instance_info.id,
-                      username, user_new, hostname)
-        # The last_http_code doesn't have to be checked, since the exception
-        # is thrown before the actual http request is executed
-        self.dbaas.users.delete(instance_info.id, username, hostname=hostname)
-
     @test(depends_on=[test_create_users])
     def test_hostname_ipv4_restriction(self):
         # By default, user hostnames are required to be % or IPv4 addresses.
@@ -443,7 +426,7 @@ class TestUsers(object):
         assert_true(len(users) <= limit)
         assert_true(users.next is not None)
         expected_marker = "%s@%s" % (users[-1].name, users[-1].host)
-        expected_marker = urllib.quote(expected_marker)
+        expected_marker = urllib_parse.quote(expected_marker)
         assert_equal(marker, expected_marker)
         marker = users.next
 
