@@ -20,13 +20,10 @@ from trove.common.exception import PollTimeOut
 from trove.common import instance as rd_instance
 from trove.tests.fakes.common import authorize
 
+import collections
 import eventlet
 import uuid
 
-try:
-    from collections import OrderedDict
-except ImportError:
-    from ordereddict import OrderedDict
 
 LOG = logging.getLogger(__name__)
 FAKE_HOSTS = ["fake_host_1", "fake_host_2"]
@@ -651,7 +648,7 @@ class FakeHosts(object):
     def __init__(self, servers):
         # Use an ordered dict to make the results of the fake api call
         # return in the same order for the example generator.
-        self.hosts = OrderedDict()
+        self.hosts = collections.OrderedDict()
         for host in FAKE_HOSTS:
             self.add_host(FakeHost(host, servers))
 
