@@ -16,8 +16,6 @@
 from trove.common import utils
 from trove.tests.api.instances import CheckInstance
 from trove.tests.scenario.helpers.test_helper import DataType
-from trove.tests.scenario import runners
-from trove.tests.scenario.runners.test_runners import SkipKnownBug
 from trove.tests.scenario.runners.test_runners import TestRunner
 from troveclient.compat import exceptions
 
@@ -32,6 +30,8 @@ class ReplicationRunner(TestRunner):
         self.replica_2_id = 0
         self.master_host = self.get_instance_host(self.master_id)
         self.replica_1_host = None
+        self.master_backup_count = None
+        self.used_data_sets = set()
 
     def run_add_data_for_replication(self, data_type=DataType.small):
         self.assert_add_replication_data(data_type, self.master_host)
@@ -41,6 +41,7 @@ class ReplicationRunner(TestRunner):
         'helper' class should implement the 'add_<data_type>_data' method.
         """
         self.test_helper.add_data(data_type, host)
+        self.used_data_sets.add(data_type)
 
     def run_verify_data_for_replication(self, data_type=DataType.small):
         self.assert_verify_replication_data(data_type, self.master_host)
@@ -54,6 +55,8 @@ class ReplicationRunner(TestRunner):
     def run_create_single_replica(self, expected_states=['BUILD', 'ACTIVE'],
                                   expected_http_code=200):
         master_id = self.instance_info.id
+        self.master_backup_count = len(
+            self.auth_client.instances.backups(master_id))
         self.replica_1_id = self.assert_replica_create(
             master_id, 'replica1', 1, expected_states, expected_http_code)
         self.replica_1_host = self.get_instance_host(self.replica_1_id)
@@ -64,7 +67,10 @@ class ReplicationRunner(TestRunner):
         replica = self.auth_client.instances.create(
             self.instance_info.name + replica_name,
             self.instance_info.dbaas_flavor_href,
-            self.instance_info.volume, slave_of=master_id,
+            self.instance_info.volume, replica_of=master_id,
+            datastore=self.instance_info.dbaas_datastore,
+            datastore_version=self.instance_info.dbaas_datastore_version,
+            nics=self.instance_info.nics,
             replica_count=replica_count)
         replica_id = replica.id
 
@@ -88,7 +94,7 @@ class ReplicationRunner(TestRunner):
     def _assert_is_replica(self, instance_id, master_id):
         instance = self.get_instance(instance_id)
         self.assert_client_code(200)
-        CheckInstance(instance._info).slave_of()
+        CheckInstance(instance._info).replica_of()
         self.assert_equal(master_id, instance._info['replica_of']['id'],
                           'Unexpected replication master ID')
 
@@ -137,12 +143,10 @@ class ReplicationRunner(TestRunner):
 
     def run_eject_valid_master(self, expected_exception=exceptions.BadRequest,
                                expected_http_code=400):
-        # self.assert_raises(
-        #     expected_exception, expected_http_code,
-        #     self.auth_client.instances.eject_replica_source,
-        #     self.instance_info.id)
-        # Uncomment once BUG_EJECT_VALID_MASTER is fixed
-        raise SkipKnownBug(runners.BUG_EJECT_VALID_MASTER)
+        self.assert_raises(
+            expected_exception, expected_http_code,
+            self.auth_client.instances.eject_replica_source,
+            self.instance_info.id)
 
     def run_delete_valid_master(self, expected_exception=exceptions.Forbidden,
                                 expected_http_code=403):
@@ -203,9 +207,9 @@ class ReplicationRunner(TestRunner):
         """In order for this to work, the corresponding datastore
         'helper' class should implement the 'remove_<type>_data' method.
         """
-        self.test_helper.remove_data(DataType.small, host)
-        self.test_helper.remove_data(DataType.tiny, host)
-        self.test_helper.remove_data(DataType.tiny2, host)
+        for data_set in self.used_data_sets:
+            self.report.log("Removing replicated data set: %s" % data_set)
+            self.test_helper.remove_data(data_set, host)
 
     def run_detach_replica_from_source(self,
                                        expected_states=['ACTIVE'],
@@ -270,3 +274,7 @@ class ReplicationRunner(TestRunner):
         replica_ids = self._get_replica_set(master_id)
         self.assert_delete_instances(replica_ids, expected_last_state,
                                      expected_http_code)
+
+    def run_test_backup_deleted(self):
+        backup = self.auth_client.instances.backups(self.master_id)
+        self.assert_equal(self.master_backup_count, len(backup))
