@@ -13,11 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-import urllib
+from six.moves.urllib import parse as urllib_parse
+
+from proboscis import SkipTest
 
 from trove.tests.scenario.runners.test_runners import TestRunner
 from troveclient.compat import exceptions
-from troveclient.openstack.common.apiclient.exceptions import ValidationError
 
 
 class UserActionsRunner(TestRunner):
@@ -28,10 +29,23 @@ class UserActionsRunner(TestRunner):
     # likely require replacing GA casts with calls which I believe are
     # more appropriate anyways.
 
+    def __init__(self):
+        super(UserActionsRunner, self).__init__()
+        self.user_defs = []
+
+    @property
+    def first_user_def(self):
+        if self.user_defs:
+            return self.user_defs[0]
+        raise SkipTest("No valid user definitions provided.")
+
     def run_users_create(self, expected_http_code=202):
         users = self.test_helper.get_valid_user_definitions()
-        self.user_defs = self.assert_users_create(
-            self.instance_info.id, users, expected_http_code)
+        if users:
+            self.user_defs = self.assert_users_create(
+                self.instance_info.id, users, expected_http_code)
+        else:
+            raise SkipTest("No valid user definitions provided.")
 
     def assert_users_create(self, instance_id, serial_users_def,
                             expected_http_code):
@@ -56,10 +70,11 @@ class UserActionsRunner(TestRunner):
 
     def _assert_user_matches(self, user, expected_user_def):
         user_name = expected_user_def['name']
-        self.assert_equal(user.name, expected_user_def['name'],
+        self.assert_equal(expected_user_def['name'], user.name,
                           "Mismatch of names for user: %s" % user_name)
-        self.assert_equal(user.databases, expected_user_def['databases'],
-                          "Mismatch of databases for user: %s" % user_name)
+        self.assert_list_elements_equal(
+            expected_user_def['databases'], user.databases,
+            "Mismatch of databases for user: %s" % user_name)
 
     def run_users_list(self, expected_http_code=200):
         self.assert_users_list(
@@ -101,8 +116,7 @@ class UserActionsRunner(TestRunner):
         self.assert_pagination_match(list_page, full_list, 0, limit)
         if marker:
             last_user = list_page[-1]
-            expected_marker = urllib.quote(
-                '%s@%s' % (last_user.name, last_user.host))
+            expected_marker = self.as_pagination_marker(last_user)
             self.assert_equal(expected_marker, marker,
                               "Pagination marker should be the last element "
                               "in the page.")
@@ -111,6 +125,9 @@ class UserActionsRunner(TestRunner):
             self.assert_pagination_match(
                 list_page, full_list, limit, len(full_list))
 
+    def as_pagination_marker(self, user):
+        return urllib_parse.quote(user.name)
+
     def run_user_create_with_no_attributes(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
@@ -154,7 +171,7 @@ class UserActionsRunner(TestRunner):
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
         self.assert_users_create_failure(
-            self.instance_info.id, self.user_defs[0],
+            self.instance_info.id, self.first_user_def,
             expected_exception, expected_http_code)
 
     def run_system_user_create(
@@ -178,25 +195,19 @@ class UserActionsRunner(TestRunner):
             expected_exception, expected_http_code,
             self.auth_client.users.create, instance_id, serial_users_def)
 
-    def run_user_update_with_no_attributes(self):
-        # Note: this is caught on the client-side.
-        self.assert_user_attribute_update_failure(
-            self.instance_info.id, self.user_defs[0],
-            {}, ValidationError, None)
-
     def run_user_update_with_blank_name(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
         self.assert_user_attribute_update_failure(
-            self.instance_info.id, self.user_defs[0], {'name': ''},
+            self.instance_info.id, self.first_user_def, {'name': ''},
             expected_exception, expected_http_code)
 
     def run_user_update_with_existing_name(
             self, expected_exception=exceptions.BadRequest,
             expected_http_code=400):
         self.assert_user_attribute_update_failure(
-            self.instance_info.id, self.user_defs[0],
-            {'name': self.user_defs[0]['name']},
+            self.instance_info.id, self.first_user_def,
+            {'name': self.first_user_def['name']},
             expected_exception, expected_http_code)
 
     def assert_user_attribute_update_failure(
@@ -227,7 +238,7 @@ class UserActionsRunner(TestRunner):
                     expected_exception, expected_http_code)
 
     def run_user_attribute_update(self, expected_http_code=202):
-        updated_def = self.user_defs[0]
+        updated_def = self.first_user_def
         # Update the name by appending a random string to it.
         updated_name = ''.join([updated_def['name'], 'upd'])
         update_attribites = {'name': updated_name,
@@ -343,3 +354,21 @@ class UserActionsRunner(TestRunner):
 
     def get_system_users(self):
         return self.get_datastore_config_property('ignore_users')
+
+
+class MysqlUserActionsRunner(UserActionsRunner):
+
+    def as_pagination_marker(self, user):
+        return urllib_parse.quote('%s@%s' % (user.name, user.host))
+
+
+class MariadbUserActionsRunner(MysqlUserActionsRunner):
+
+    def __init__(self):
+        super(MariadbUserActionsRunner, self).__init__()
+
+
+class PerconaUserActionsRunner(MysqlUserActionsRunner):
+
+    def __init__(self):
+        super(PerconaUserActionsRunner, self).__init__()
