@@ -16,26 +16,27 @@ import hashlib
 import mock
 import os
 
-from mock import Mock, MagicMock, patch, ANY
+from mock import Mock, MagicMock, patch, ANY, DEFAULT
 from oslo_utils import netutils
 from webob.exc import HTTPNotFound
 
 from trove.backup.state import BackupState
 from trove.common.context import TroveContext
+from trove.common.strategies.storage.base import Storage
 from trove.common import utils
 from trove.conductor import api as conductor_api
 from trove.guestagent.backup import backupagent
 from trove.guestagent.common import configuration
-from trove.guestagent.datastore.experimental.mongodb.service import MongoDBApp
+from trove.guestagent.common.configuration import ImportOverrideStrategy
 from trove.guestagent.strategies.backup.base import BackupRunner
 from trove.guestagent.strategies.backup.base import UnknownBackupType
 from trove.guestagent.strategies.backup.experimental import couchbase_impl
+from trove.guestagent.strategies.backup.experimental import db2_impl
 from trove.guestagent.strategies.backup.experimental import mongo_impl
 from trove.guestagent.strategies.backup.experimental import redis_impl
 from trove.guestagent.strategies.backup import mysql_impl
 from trove.guestagent.strategies.backup.mysql_impl import MySqlApp
 from trove.guestagent.strategies.restore.base import RestoreRunner
-from trove.guestagent.strategies.storage.base import Storage
 from trove.tests.unittests import trove_testtools
 
 
@@ -43,7 +44,7 @@ def create_fake_data():
     from random import choice
     from string import ascii_letters
 
-    return ''.join([choice(ascii_letters) for _ in xrange(1024)])
+    return ''.join([choice(ascii_letters) for _ in range(1024)])
 
 
 class MockBackup(BackupRunner):
@@ -168,6 +169,15 @@ class BackupAgentTest(trove_testtools.TestCase):
 
     def setUp(self):
         super(BackupAgentTest, self).setUp()
+        self.patch_ope = patch.multiple('os.path',
+                                        exists=DEFAULT)
+        self.mock_ope = self.patch_ope.start()
+        self.addCleanup(self.patch_ope.stop)
+        self.patch_pc = patch('trove.guestagent.datastore.service.'
+                              'BaseDbStatus.prepare_completed')
+        self.mock_pc = self.patch_pc.start()
+        self.mock_pc.__get__ = Mock(return_value=True)
+        self.addCleanup(self.patch_pc.stop)
         self.get_auth_pwd_patch = patch.object(
             MySqlApp, 'get_auth_password', MagicMock(return_value='123'))
         self.get_auth_pwd_mock = self.get_auth_pwd_patch.start()
@@ -191,7 +201,7 @@ class BackupAgentTest(trove_testtools.TestCase):
 
     def test_backup_impl_MySQLDump(self):
         """This test is for
-           guestagent/strategies/backup/impl
+           guestagent/strategies/backup/mysql_impl
         """
         mysql_dump = mysql_impl.MySQLDump(
             'abc', extra_opts='')
@@ -214,7 +224,7 @@ class BackupAgentTest(trove_testtools.TestCase):
         MySqlApp, 'get_data_dir', return_value='/var/lib/mysql/data')
     def test_backup_impl_InnoBackupEx(self, mock_datadir):
         """This test is for
-           guestagent/strategies/backup/impl
+           guestagent/strategies/backup/mysql_impl
         """
         inno_backup_ex = mysql_impl.InnoBackupEx('innobackupex', extra_opts='')
         self.assertIsNotNone(inno_backup_ex.cmd)
@@ -242,7 +252,18 @@ class BackupAgentTest(trove_testtools.TestCase):
         self.assertIsNotNone(cbbackup.manifest)
         self.assertIn('gz.enc', cbbackup.manifest)
 
-    @mock.patch.object(MongoDBApp, '_init_overrides_dir')
+    def test_backup_impl_DB2Backup(self):
+        netutils.get_my_ipv4 = Mock(return_value="1.1.1.1")
+        db2_backup = db2_impl.DB2Backup('db2backup', extra_opts='')
+        self.assertIsNotNone(db2_backup)
+        str_db2_backup_cmd = ("sudo tar cPf - /home/db2inst1/db2inst1/backup "
+                              "| gzip | openssl enc -aes-256-cbc -salt -pass "
+                              "pass:default_aes_cbc_key")
+        self.assertEqual(str_db2_backup_cmd, db2_backup.cmd)
+        self.assertIsNotNone(db2_backup.manifest)
+        self.assertIn('gz.enc', db2_backup.manifest)
+
+    @mock.patch.object(ImportOverrideStrategy, '_initialize_import_directory')
     def test_backup_impl_MongoDump(self, _):
         netutils.get_my_ipv4 = Mock(return_value="1.1.1.1")
         utils.execute_with_timeout = Mock(return_value=None)
@@ -255,7 +276,7 @@ class BackupAgentTest(trove_testtools.TestCase):
         self.assertIsNotNone(mongodump.manifest)
         self.assertIn('gz.enc', mongodump.manifest)
 
-    @patch.object(utils, 'execute_with_timeout')
+    @patch.object(utils, 'execute_with_timeout', return_value=('0', ''))
     @patch.object(configuration.ConfigurationManager, 'parse_configuration',
                   Mock(return_value={'dir': '/var/lib/redis',
                                      'dbfilename': 'dump.rdb'}))
@@ -374,7 +395,8 @@ class BackupAgentTest(trove_testtools.TestCase):
     @patch.object(conductor_api.API, 'get_client', Mock(return_value=Mock()))
     @patch.object(conductor_api.API, 'update_backup',
                   Mock(return_value=Mock()))
-    def test_execute_lossy_backup(self):
+    @patch('trove.guestagent.backup.backupagent.LOG')
+    def test_execute_lossy_backup(self, mock_logging):
         """This test verifies that incomplete writes to swift will fail."""
         with patch.object(MockSwift, 'save',
                           return_value=(False, 'Error', 'y', 'z')):
@@ -421,7 +443,8 @@ class BackupAgentTest(trove_testtools.TestCase):
                                       bkup_info,
                                       '/var/lib/mysql/data')
 
-    def test_restore_unknown(self):
+    @patch('trove.guestagent.backup.backupagent.LOG')
+    def test_restore_unknown(self, mock_logging):
         with patch.object(backupagent, 'get_restore_strategy',
                           side_effect=ImportError):
 
@@ -442,7 +465,8 @@ class BackupAgentTest(trove_testtools.TestCase):
     @patch.object(MockSwift, 'load_metadata', return_value={'lsn': '54321'})
     @patch.object(MockStorage, 'save_metadata')
     @patch.object(backupagent, 'get_storage_strategy', return_value=MockSwift)
-    def test_backup_incremental_metadata(self,
+    @patch('trove.guestagent.backup.backupagent.LOG')
+    def test_backup_incremental_metadata(self, mock_logging,
                                          get_storage_strategy_mock,
                                          save_metadata_mock,
                                          load_metadata_mock,
@@ -462,7 +486,9 @@ class BackupAgentTest(trove_testtools.TestCase):
                          'location': 'fake-location',
                          'type': 'InnoBackupEx',
                          'checksum': 'fake-checksum',
-                         'parent': {'location': 'fake', 'checksum': 'md5'}
+                         'parent': {'location': 'fake', 'checksum': 'md5'},
+                         'datastore': 'mysql',
+                         'datastore_version': 'bo.gus'
                          }
 
             agent.execute_backup(TroveContext(),
