@@ -52,7 +52,8 @@ class TestConfigurationManager(trove_testtools.TestCase):
             override_strategy=sample_strategy)
 
         manager.parse_configuration()
-        read_file.assert_called_with(sample_path, codec=sample_codec)
+        read_file.assert_called_with(sample_path, codec=sample_codec,
+                                     as_root=sample_requires_root)
 
         with patch.object(manager, 'parse_configuration',
                           return_value={'key1': 'v1', 'key2': 'v2'}):
@@ -188,6 +189,7 @@ class TestConfigurationOverrideStrategy(trove_testtools.TestCase):
             [system_overrides_v1, system_overrides_v2],
             [user_overrides_v1, user_overrides_v2], False)
 
+    @patch.multiple(operating_system, chmod=Mock(), chown=Mock())
     def _test_import_override_strategy(
             self, system_overrides, user_overrides, test_multi_rev):
         base_config_contents = {'Section_1': {'name': 'pi',
@@ -356,6 +358,7 @@ class TestConfigurationOverrideStrategy(trove_testtools.TestCase):
         self._assert_get_value(ImportOverrideStrategy(revision_dir, 'ext'))
         self._assert_get_value(OneFileOverrideStrategy(revision_dir))
 
+    @patch.multiple(operating_system, chmod=Mock(), chown=Mock())
     def _assert_get_value(self, override_strategy):
         base_config_contents = {'Section_1': {'name': 'pi',
                                               'is_number': 'True',
