@@ -11,27 +11,36 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from mock import DEFAULT
 from mock import MagicMock
 from mock import patch
+from os import path
 from testtools.matchers import Is
-import trove_testtools
 
-from trove.common.context import TroveContext
 from trove.common.exception import DatastoreOperationNotSupported
 from trove.common import instance as rd_instance
+from trove.guestagent.common import configuration
+from trove.guestagent.common.configuration import ImportOverrideStrategy
+from trove.guestagent.common import operating_system
 from trove.guestagent.datastore.experimental.vertica.manager import Manager
 from trove.guestagent.datastore.experimental.vertica.service import (
     VerticaAppStatus)
 from trove.guestagent.datastore.experimental.vertica.service import VerticaApp
+from trove.guestagent.datastore.experimental.vertica import system
 from trove.guestagent import dbaas
 from trove.guestagent import volume
 from trove.guestagent.volume import VolumeDevice
+from trove.tests.unittests import trove_testtools
 
 
 class GuestAgentManagerTest(trove_testtools.TestCase):
-    def setUp(self):
+
+    @patch.object(ImportOverrideStrategy, '_initialize_import_directory')
+    @patch.multiple(operating_system, exists=DEFAULT, write_file=DEFAULT,
+                    chown=DEFAULT, chmod=DEFAULT)
+    def setUp(self, *args, **kwargs):
         super(GuestAgentManagerTest, self).setUp()
-        self.context = TroveContext()
+        self.context = trove_testtools.TroveTestContext(self)
         self.manager = Manager()
         self.origin_format = volume.VolumeDevice.format
         self.origin_migrate_data = volume.VolumeDevice.migrate_data
@@ -45,11 +54,11 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
         self.origin_start_db = VerticaApp.start_db
         self.origin_restart = VerticaApp.restart
         self.origin_install_if = VerticaApp.install_if_needed
-        self.origin_complete_install = VerticaApp.complete_install_or_restart
         self.origin_enable_root = VerticaApp.enable_root
         self.origin_is_root_enabled = VerticaApp.is_root_enabled
         self.origin_prepare_for_install_vertica = (
             VerticaApp.prepare_for_install_vertica)
+        self.origin_add_udls = VerticaApp.add_udls
 
     def tearDown(self):
         super(GuestAgentManagerTest, self).tearDown()
@@ -65,11 +74,11 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
         VerticaApp.start_db = self.origin_start_db
         VerticaApp.restart = self.origin_restart
         VerticaApp.install_if_needed = self.origin_install_if
-        VerticaApp.complete_install_or_restart = self.origin_complete_install
         VerticaApp.enable_root = self.origin_enable_root
         VerticaApp.is_root_enabled = self.origin_is_root_enabled
         VerticaApp.prepare_for_install_vertica = (
             self.origin_prepare_for_install_vertica)
+        VerticaApp.add_udls = self.origin_add_udls
 
     def test_update_status(self):
         mock_status = MagicMock()
@@ -77,6 +86,8 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
         self.manager.update_status(self.context)
         mock_status.update.assert_any_call()
 
+    @patch.object(path, 'exists', MagicMock())
+    @patch.object(configuration.ConfigurationManager, 'save_configuration')
     def _prepare_dynamic(self, packages,
                          config_content='MockContent', device_path='/dev/vdb',
                          backup_id=None,
@@ -94,7 +105,6 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
         self.manager.appStatus = mock_status
 
         mock_status.begin_install = MagicMock(return_value=None)
-        path_exists_function = MagicMock(return_value=True)
         volume.VolumeDevice.format = MagicMock(return_value=None)
         volume.VolumeDevice.migrate_data = MagicMock(return_value=None)
         volume.VolumeDevice.mount = MagicMock(return_value=None)
@@ -108,7 +118,7 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
         VerticaApp.install_vertica = MagicMock(return_value=None)
         VerticaApp.create_db = MagicMock(return_value=None)
         VerticaApp.prepare_for_install_vertica = MagicMock(return_value=None)
-        VerticaApp.complete_install_or_restart = MagicMock(return_value=None)
+        VerticaApp.add_udls = MagicMock()
         # invocation
         self.manager.prepare(context=self.context, packages=packages,
                              config_contents=config_content,
@@ -118,8 +128,7 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
                              mount_point="/var/lib/vertica",
                              backup_info=backup_info,
                              overrides=None,
-                             cluster_config=None,
-                             path_exists_function=path_exists_function)
+                             cluster_config=None)
 
         self.assertEqual(expected_vol_count, VolumeDevice.format.call_count)
         self.assertEqual(expected_vol_count,
@@ -135,7 +144,7 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
         VerticaApp.prepare_for_install_vertica.assert_any_call()
         VerticaApp.install_vertica.assert_any_call()
         VerticaApp.create_db.assert_any_call()
-        VerticaApp.complete_install_or_restart.assert_any_call()
+        VerticaApp.add_udls.assert_any_call()
 
     def test_prepare_pkg(self):
         self._prepare_dynamic(['vertica'])
@@ -164,22 +173,47 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
     @patch.object(VerticaApp, 'install_vertica')
     @patch.object(VerticaApp, '_export_conf_to_members')
     @patch.object(VerticaApp, 'create_db')
-    def test_install_cluster(self, mock_install, mock_export, mock_create_db):
+    @patch.object(VerticaApp, 'add_udls')
+    def test_install_cluster(self, mock_udls, mock_install, mock_export,
+                             mock_create_db):
         members = ['test1', 'test2']
         self.manager.install_cluster(self.context, members)
         mock_install.assert_called_with('test1,test2')
         mock_export.assert_called_with(members)
         mock_create_db.assert_called_with('test1,test2')
+        mock_udls.assert_any_call()
 
     @patch.object(VerticaAppStatus, 'set_status')
     @patch.object(VerticaApp, 'install_cluster',
                   side_effect=RuntimeError("Boom!"))
-    def test_install_cluster_failure(self, mock_install, mock_set_status):
+    @patch('trove.guestagent.datastore.experimental.vertica.manager.LOG')
+    def test_install_cluster_failure(self, mock_logging,
+                                     mock_install, mock_set_status):
         members = ["test1", "test2"]
         self.assertRaises(RuntimeError, self.manager.install_cluster,
                           self.context, members)
         mock_set_status.assert_called_with(rd_instance.ServiceStatuses.FAILED)
 
+    @patch.object(VerticaApp, '_get_database_password')
+    @patch.object(path, 'isfile')
+    @patch.object(system, 'exec_vsql_command')
+    def test_add_udls(self, mock_vsql, mock_isfile, mock_pwd):
+        mock_vsql.return_value = (None, None)
+        password = 'password'
+        mock_pwd.return_value = password
+        mock_isfile.return_value = True
+        self.manager.app.add_udls()
+        mock_vsql.assert_any_call(
+            password,
+            "CREATE LIBRARY curllib AS "
+            "'/opt/vertica/sdk/examples/build/cURLLib.so'"
+        )
+        mock_vsql.assert_any_call(
+            password,
+            "CREATE SOURCE curl AS LANGUAGE 'C++' NAME 'CurlSourceFactory' "
+            "LIBRARY curllib"
+        )
+
     @patch.object(volume.VolumeDevice, 'mount_points', return_value=[])
     @patch.object(volume.VolumeDevice, 'unmount_device', return_value=None)
     @patch.object(volume.VolumeDevice, 'mount', return_value=None)
@@ -187,6 +221,7 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
     @patch.object(volume.VolumeDevice, 'format', return_value=None)
     @patch.object(VerticaApp, 'prepare_for_install_vertica')
     @patch.object(VerticaApp, 'install_if_needed')
+    @patch.object(VerticaApp, 'add_udls')
     @patch.object(VerticaAppStatus, 'begin_install')
     def _prepare_method(self, instance_id, instance_type, *args):
         cluster_config = {"id": instance_id,
@@ -204,24 +239,21 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
     def test_prepare_member(self, mock_set_status):
         self._prepare_method("test-instance-3", "member")
         mock_set_status.assert_called_with(
-            rd_instance.ServiceStatuses.BUILD_PENDING)
-
-    def test_reset_configuration(self):
-        try:
-            configuration = {'config_contents': 'some junk'}
-            self.manager.reset_configuration(self.context, configuration)
-        except Exception:
-            self.fail("reset_configuration raised exception unexpectedly.")
+            rd_instance.ServiceStatuses.INSTANCE_READY, force=True)
 
     def test_rpc_ping(self):
         output = self.manager.rpc_ping(self.context)
         self.assertTrue(output)
 
     @patch.object(VerticaAppStatus, 'set_status')
-    def test_prepare_invalid_cluster_config(self, mock_set_status):
-        self._prepare_method("test-instance-3", "query_router")
+    @patch('trove.guestagent.datastore.manager.LOG')
+    def test_prepare_invalid_cluster_config(self, mock_logging,
+                                            mock_set_status):
+        self.assertRaises(RuntimeError,
+                          self._prepare_method,
+                          "test-instance-3", "query_router")
         mock_set_status.assert_called_with(
-            rd_instance.ServiceStatuses.FAILED)
+            rd_instance.ServiceStatuses.FAILED, force=True)
 
     def test_get_filesystem_stats(self):
         with patch.object(dbaas, 'get_filesystem_volume_stats'):
@@ -249,15 +281,15 @@ class GuestAgentManagerTest(trove_testtools.TestCase):
             test_resize_fs = volume.VolumeDevice.resize_fs.call_args_list[0]
             test_resize_fs.assert_called_with('/var/lib/vertica')
 
-    def test_cluster_complete(self):
-        mock_status = MagicMock()
-        mock_status.set_status = MagicMock()
-        self.manager.appStatus = mock_status
-        mock_status._get_actual_db_status = MagicMock(
+    @patch.object(operating_system, 'write_file')
+    def test_cluster_complete(self, mock_write_file):
+        mock_set_status = MagicMock()
+        self.manager.appStatus.set_status = mock_set_status
+        self.manager.appStatus._get_actual_db_status = MagicMock(
             return_value=rd_instance.ServiceStatuses.RUNNING)
         self.manager.cluster_complete(self.context)
-        mock_status.set_status.assert_called_with(
-            rd_instance.ServiceStatuses.RUNNING)
+        mock_set_status.assert_called_with(
+            rd_instance.ServiceStatuses.RUNNING, force=True)
 
     def test_get_public_keys(self):
         with patch.object(VerticaApp, 'get_public_keys',
