@@ -96,29 +96,32 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
 
     @patch.object(ClusterTasks, 'update_statuses_on_failure')
     @patch.object(InstanceServiceStatus, 'find_by')
-    def test_all_instances_ready_bad_status(self,
+    @patch('trove.taskmanager.models.LOG')
+    def test_all_instances_ready_bad_status(self, mock_logging,
                                             mock_find, mock_update):
         (mock_find.return_value.
          get_status.return_value) = ServiceStatuses.FAILED
         ret_val = self.clustertasks._all_instances_ready(["1", "2", "3", "4"],
                                                          self.cluster_id)
         mock_update.assert_called_with(self.cluster_id, None)
-        self.assertEqual(False, ret_val)
+        self.assertFalse(ret_val)
 
     @patch.object(InstanceServiceStatus, 'find_by')
     def test_all_instances_ready(self, mock_find):
         (mock_find.return_value.
-         get_status.return_value) = ServiceStatuses.BUILD_PENDING
+         get_status.return_value) = ServiceStatuses.INSTANCE_READY
         ret_val = self.clustertasks._all_instances_ready(["1", "2", "3", "4"],
                                                          self.cluster_id)
-        self.assertEqual(True, ret_val)
+        self.assertTrue(ret_val)
 
     @patch.object(ClusterTasks, 'update_statuses_on_failure')
     @patch.object(ClusterTasks, 'get_guest')
     @patch.object(ClusterTasks, 'get_ip')
     @patch.object(datastore_models.Datastore, 'load')
     @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
-    def test_init_replica_set_failure(self, mock_dv, mock_ds,
+    @patch(
+        'trove.common.strategies.cluster.experimental.mongodb.taskmanager.LOG')
+    def test_init_replica_set_failure(self, mock_logging, mock_dv, mock_ds,
                                       mock_ip, mock_guest,
                                       mock_update):
         member1 = BaseInstance(Mock(), self.dbinst1, Mock(),
@@ -138,7 +141,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
                                                       other_members)
 
         mock_update.assert_called_with(self.cluster_id, shard_id='shard-1')
-        self.assertEqual(False, ret_val)
+        self.assertFalse(ret_val)
 
     @patch.object(ClusterTasks, 'get_guest')
     @patch.object(ClusterTasks, 'get_ip')
@@ -161,7 +164,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
         mock_guest.return_value.add_members.assert_called_with(
             ["10.0.0.3"]
         )
-        self.assertEqual(True, ret_val)
+        self.assertTrue(ret_val)
 
     @patch.object(ClusterTasks, 'update_statuses_on_failure')
     @patch.object(ClusterTasks, '_init_replica_set')
@@ -169,8 +172,11 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
     @patch.object(ClusterTasks, 'get_ip')
     @patch.object(datastore_models.Datastore, 'load')
     @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
-    def test_create_shard_failure(self, mock_dv, mock_ds, mock_ip,
-                                  mock_guest, mock_init_rs, mock_update):
+    @patch(
+        'trove.common.strategies.cluster.experimental.mongodb.taskmanager.LOG')
+    def test_create_shard_failure(self, mock_logging, mock_dv, mock_ds,
+                                  mock_ip, mock_guest, mock_init_rs,
+                                  mock_update):
         member1 = BaseInstance(Mock(), self.dbinst1, Mock(),
                                InstanceServiceStatus(ServiceStatuses.NEW))
         member2 = BaseInstance(Mock(), self.dbinst2, Mock(),
@@ -190,7 +196,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
 
         mock_init_rs.assert_called_with(member1, [member2])
         mock_update.assert_called_with(self.cluster_id, shard_id="shard-1")
-        self.assertEqual(False, ret_val)
+        self.assertFalse(ret_val)
 
     @patch.object(ClusterTasks, '_init_replica_set')
     @patch.object(ClusterTasks, 'get_guest')
@@ -218,7 +224,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
 
         mock_init_rs.assert_called_with(member1, [member2])
         mock_add_shard.assert_called_with("testrs", "10.0.0.2")
-        self.assertEqual(True, ret_val)
+        self.assertTrue(ret_val)
 
     @patch.object(ClusterTasks, 'reset_task')
     @patch.object(ClusterTasks, 'get_guest')
@@ -320,12 +326,11 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
     @patch.object(ClusterTasks, 'get_cluster_admin_password')
     @patch.object(datastore_models.Datastore, 'load')
     @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
+    @patch(
+        'trove.common.strategies.cluster.experimental.mongodb.taskmanager.LOG')
     def test_add_query_routers_failure(self,
-                                       mock_dv,
-                                       mock_ds,
-                                       mock_password,
-                                       mock_guest,
-                                       mock_update):
+                                       mock_logging, mock_dv, mock_ds,
+                                       mock_password, mock_guest, mock_update):
         query_router = BaseInstance(
             Mock(), self.dbinst3, Mock(),
             InstanceServiceStatus(ServiceStatuses.NEW)
@@ -335,29 +340,28 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
         ret_val = self.clustertasks._add_query_routers([query_router],
                                                        ['10.0.0.5'])
         mock_update.assert_called_with(self.cluster_id)
-        self.assertEqual(False, ret_val)
+        self.assertFalse(ret_val)
 
     @patch.object(ClusterTasks, 'get_guest')
-    @patch.object(ClusterTasks, 'get_cluster_admin_password')
     @patch.object(datastore_models.Datastore, 'load')
     @patch.object(datastore_models.DatastoreVersion, 'load_by_uuid')
     def test_add_query_routers(self,
                                mock_dv,
                                mock_ds,
-                               mock_password,
                                mock_guest):
+        password = 'pwd'
         query_router = BaseInstance(
             Mock(), self.dbinst3, Mock(),
             InstanceServiceStatus(ServiceStatuses.NEW)
         )
-        mock_password.return_value = 'pwd'
 
         ret_val = self.clustertasks._add_query_routers([query_router],
-                                                       ['10.0.0.5'])
+                                                       ['10.0.0.5'],
+                                                       admin_password=password)
         mock_guest.assert_called_with(query_router)
         mock_guest().add_config_servers.assert_called_with(['10.0.0.5'])
-        mock_guest().store_admin_password.assert_called_with('pwd')
-        self.assertEqual(True, ret_val)
+        mock_guest().store_admin_password.assert_called_with(password)
+        self.assertTrue(ret_val)
 
     @patch.object(ClusterTasks, 'get_guest')
     @patch.object(utils, 'generate_random_password')
@@ -366,21 +370,21 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
     def test_add_query_routers_new_cluster(self,
                                            mock_dv,
                                            mock_ds,
-                                           mock_password,
+                                           mock_gen_password,
                                            mock_guest):
+        password = 'pwd'
         query_router = BaseInstance(
             Mock(), self.dbinst3, Mock(),
             InstanceServiceStatus(ServiceStatuses.NEW)
         )
-        mock_password.return_value = 'pwd'
+        mock_gen_password.return_value = password
 
         ret_val = self.clustertasks._add_query_routers([query_router],
-                                                       ['10.0.0.5'],
-                                                       new_cluster=True)
+                                                       ['10.0.0.5'])
         mock_guest.assert_called_with(query_router)
         mock_guest().add_config_servers.assert_called_with(['10.0.0.5'])
-        mock_guest().create_admin_user.assert_called_with('pwd')
-        self.assertEqual(True, ret_val)
+        mock_guest().create_admin_user.assert_called_with(password)
+        self.assertTrue(ret_val)
 
     @patch.object(ClusterTasks, 'reset_task')
     @patch.object(ClusterTasks, 'get_guest')
@@ -406,6 +410,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
         mock_reset_task.assert_called_with()
 
     @patch.object(ClusterTasks, '_add_query_routers')
+    @patch.object(ClusterTasks, 'get_cluster_admin_password')
     @patch.object(ClusterTasks, 'get_ip')
     @patch.object(Instance, 'load')
     @patch.object(datastore_models.Datastore, 'load')
@@ -415,6 +420,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
                                        mock_ds,
                                        mock_load,
                                        mock_ip,
+                                       mock_get_password,
                                        mock_add_query_router):
         query_router = BaseInstance(
             Mock(), self.dbinst3, Mock(),
@@ -431,7 +437,7 @@ class MongoDbClusterTasksTest(trove_testtools.TestCase):
         self._run_grow_cluster(new_instances_ids=[query_router.id])
 
         mock_add_query_router.assert_called_with(
-            [query_router], ['10.0.0.5']
+            [query_router], ['10.0.0.5'], admin_password=mock_get_password()
         )
 
     @patch.object(ClusterTasks, '_create_shard')
